/*****************************************************************************/
/*   Copyright 1998 National Instruments Corporation.  All Rights Reserved.  */
/*****************************************************************************/

#include <formatio.h>
#include <visa.h>
#include <stdlib.h>  
#include <stdio.h>  
#include <math.h>
#include <string.h>
#include "avr3132s.h"

#define AVR3132S_REVISION  "Rev 4.1, 8/2001, CVI 5.0.1" /*Instrument driver revision*/
#define BUFFER_SIZE    512L   /* File I/O buffer size */

/*= ADVANTEST R3131 Series Spectrum Analyzers: ==============================*/
/*                                                                           */
/* LabWindows/CVI 5.0.1 Instrument Driver                                    */
/* Original Release: August 1998                                             */
/* By: Denis BIRINGER, Advantest Europe R&D S.A.R.L. Les ULIS France         */
/*     Fax + 33 1 69 18 25 90                                                */
/*     e-mail: d.biringer@erd.advantest.de                                   */
/*                                                                           */
/* National Instruments, Austin, TX, USA                                     */
/*     PH. 1-800-433-3488   Fax 512-794-5678                                 */
/*                                                                           */
/* Originally written in C                                                   */
/*                                                                           */
/* Modification History:                                                     */
/*                                                                           */
/* September 1998   Sends an empty string before the first read or write     */
/*                  when an RS-232 (Serial) interface is used.               */ 
/*                  Modified by: Denis BIRINGER, Advantest Europe R&D.       */
/*                                                                           */
/* November 1998    Function init modified : it is now useless to read the   */
/*                  display line, the reference line and the level window    */
/*                  status at the remote control beginning                   */ 
/*                  Function Set_Scale modified : the reference unit may     */
/*                  changed when the scale is altered                        */
/*                  Function Set_Frequency modified : when the start or stop */
/*                  frequency is modified, the span may become null or       */
/*                  become different than zero                               */
/*                  Function Read_FreqCounter modified : it now also returns */
/*                  the frequency counter resolution                         */
/*                  Function Read_Trigger modified : it now also returns the */
/*                  trigger slope                                            */
/*                  Function copy_Bitmap modified : a new type of printer    */
/*                  and paper size can now be chosen by the user             */
/*                  New Functions for antenna factor correction              */       
/*                  Modified by: Denis BIRINGER, Advantest Europe R&D.       */ 
/*                                                                           */
/* June 1999    	Functions Tracking Generator added        				 */
/*                  Modified by : Denis BIRINGER, Advantest Europe R&D       */
/*                                                                           */
/* October 1999     The driver can now be also used with the R3132 and R3162 */
/*                  Modified by : Denis BIRINGER, Advantest Europe R&D       */  
/*                                                                           */
/* January 2001     The driver can now be also used with the R3172 and R3182 */
/*					Modified by : Hitoshi Takahashi Advantest Gyoda R&D		 */
/*																			 */
/* August 2001		Mixer Functions are added to the Driver.				 */
/*					Modified by : Hitoshi Takahashi Advantest Gunma R&D		 */
/*===========================================================================*/

/*****************************************************************************/
/* INSTRUMENT-DEPENDENT COMMAND ARRAYS                                       */
/*****************************************************************************/

/* init and reset */
static ViString s_init[] = {"DL0","*IDN?",""};
static ViString s_reset[]={"IP"};

/* frequency */
static ViString s_freqType[] = {"CF", "FA", "FB"};         
static ViString s_spanType[] = {"SP", "FS", "ZS", "PKZOOM", "LTSP"};
static ViString s_readFreq[]= {"CF?","SP?","FA?","FB?"}; 

static ViString s_freqOffset[] = {"FO OFF","FO"}; 
static ViString s_readFreqOffset[] = {"FOON?","FO?"};
    
static ViString s_freqStepSize[] = {"CS","CA"};
static ViString s_readFreqStepSize[] = {"CS?","CA?"};

static ViString s_counterResolution[] = {"CN0","CN1","CN2","CN3"};
static ViString s_counterONOFF[] = {"COUNT OFF","COUNT ON"};
static ViString s_readFreqCounter[] = {"COUNT?","CNRES?","CNORD?","CN?"}; 

/* channel setting*/
static ViString s_inputType[] = {"FINPMD FREQ", "FINPMD CH1", "FINPMD CH2"};
static ViString s_editTable[] = {"CHED1", "CHED2", "CHED3"};
static ViString s_EnabeDisable[] = {"DSBL", "ENBL"};

/* reference level and unit */
static ViString s_refLevel[] = {"RL"};
static ViString s_readRefLevel[]={"RL?"};

static ViString s_refOffset[] = {"ROF","RO OFF","RON"};
static ViString s_readRefOffset[]={"RO?","ROON?"}; 

static ViString s_refUnit[] = {"AUNITS DBM", "AUNITS DBMV", "AUNITS DBUV", "AUNITS V", "AUNITS W"};
static ViString s_readRefUnit[]={"AUNITS?"}; 
static ViString s_readUnitName[] = {"dBm","dBmV","dBuV","","","V","W"};

/* scale and attenuation */
static ViString s_yScale[] = {"DD10DB","DD5DB","DD2DB","DD1DB","LL1","DD?"};
static ViString s_readScale[] = {"10dB/","5dB/","2dB/","1dB/","LIN"};   

static ViString s_att[] = {"AT","AA"};
static ViString s_readAtt[] = {"AT?","AA?"};

static ViString s_hiSense[] = {"HS OFF", "HS"};

static ViString s_impedance[] = {"OHM75" "OHM50"};
    
/* RBW and VBW */
static ViString s_RBW[] = {"BA","RB3MZ","RB1MZ","RB300KZ","RB100KZ","RB30KZ","RB10KZ","RB3KZ","RB1KZ"};                    
static ViString s_VBW[] = {"VA","VB3MZ","VB1MZ","VB300KZ","VB100KZ","VB30KZ","VB10KZ","VB3KZ","VB1KZ","VB300HZ","VB100HZ","VB30HZ","VB10HZ"};                
static ViString s_All_Coupled[] = {"AL"};
static ViString s_readRBWandVBW[] = {"VB?","RB?","VA?","BA?"};
static ViString s_wideRBW[] = {"WRBW OFF", "WRBW ON"};

/* Set Auto Manual */
static ViString s_setAuto[] = {"BA", "VA", "AS", "CA", "MPA"};
static ViString s_setManual[] = {"RB", "VB", "SW", "CS", "MPM"};
static ViString s_unitManual[] = {"HZ", "HZ", "SC", "HZ", ""};

/* sweep time, sweep mode and trigger */
static ViString s_sweepTime[] = {"SW","AS"};
static ViString s_readSweepTime[] = {"SW?","AS?"};

static ViString s_sweepMode[] = {"SI","SN","SR","TS"}; 
static ViString s_windowSweep[] = {"WDOSWP OFF","WDOSWP ON"};
static ViString s_gatedSweep[] = {"GTSWP OFF", "GTSWP ON"};
static ViString s_readSweepMode[] = {"SWM?","Continuous&full","Single&full","Continuous&window","Single&window"};

static ViString s_trigger[] = {"FR","LI","VI","TRIGSLP-","TRIGSLP+","EX","TRGSRC FREE","TRGSRC LINE"};
static ViString s_triggerTV[] = {"TRGSRC TVV","TRGSRC TVH","TVHNT","TVHPS","TVH","TRIGSLP-","TRIGSLP+"};  
static ViString s_readTrigger[] = {"TM?","TRGSRC?","TRIGSLP?","VI?","TVH?","EX?"};
static ViString s_triggerName[] = {"Free Run","Line","Video","TV-V","TV-H","External"};

/* trace */
static ViString s_traceMode[] = {"BW", "BV", "BB", "BM", "", "BMIN", "", "BGR",
								 "AW", "AV", "AB", "AM", "", "AMIN", "", "AGR",
						         "BW", "BV", "BB", "BMAX ON", "BMAX OFF", "BMIN ON", "BMIN OFF", "BGR",
						         "AW", "AV", "AB", "AMAX ON", "AMAX OFF", "AMIN ON", "AMIN OFF", "AGR"};
static ViString s_averageMode[] = {"BGSGL","BGCNT","BG","AGSGL","AGCNT","AG"};  
static ViString s_detectorMode[] = {"DETB NRM","DETB POS","DETB NEG","DETB SMP","DET NRM","DET POS","DET NEG","DET SMP"};
static ViString s_readDetectorMode[] = {"DETB?","DET?","B_Normal","B_Positive","B_Negative","B_Sample",
                                        "A_Normal","A_Positive","A_Negative","A_Sample"}; 
static ViString s_readTraceStatus[] = {"TB?","TA?"};
static ViString s_readTraceMessage[] = {"B_Write","B_View","B_Blank","B_MaxHold","B_MinHold","B_Averaging",
                                        "A_Write","A_View","A_Blank","A_MaxHold","A_MinHold","A_Averaging"};
static ViString s_traceAverage[]  = {"BGR","BGS","BGP","BGC","AGR","AGS","AGP","AGC"};
static ViString s_traceMath[] = {"BSTORE", "ASTORE", "ABA", "BAA","ADLA"};

static ViString s_tracePoints[] = {"TPS","TPL","TP?"};
static ViString s_traceBinary[] = {"TBB","TBA"};
static ViString s_traceASCII[] = {"TAB","TAA"};
static ViString s_queryBinary[] = {"TBB?","TBA?"};
static ViString s_queryASCII[] = {"TAB?","TAA?"};

/* markers */
static ViString s_mkrNormal[] = {"MKN"};
static ViString s_mkrDelta[] = {"MKD"};
static ViString s_deleteMarker[] = {"MO","MLT OFF"};
static ViString s_mkrStatusRead[] = {"MN?"};
static ViString s_normalMkrRead[] = {"MF?","ML?","MDF1?","MDL1?"};
static ViString s_deltaMkrRead[] = {"MDF2?","MDL2?"};
static ViString s_mkrStatus[] = {"markers inactive","only normal marker active",
                                 "normal and delta markers active"};
static ViString s_mkrTrace[] = {"MKTRACE TRB","MKTRACE TRA"};
static ViString s_readMkrTrace[] = {"MKTRACE?","blank","Trace A","Trace B"}; 
static ViString s_mkrOptions[] = {"MPM","MPA","FXF","FXN","CPLMK OFF","CPLMK ON"};
static ViString s_markerStepSize[] = {"MKMKS","MPM?"};
static ViString s_readMkrOptions[] = {"MPA?","MPM?","FX?","CPLMK?"};
static ViString s_markerRead[] = {"DC0", "DC1", "DC2"};
static ViString s_contDown[] = {"CDB OFF", "CDB ON"};
static ViString s_multiMkrStatus[] = {"MLT ON"}; 
static ViString s_multiMkrON[] = {"MLN1","MLN2","MLN3","MLN4","MLN5","MLN6","MLN7","MLN8","MLN9","MLN10"};
static ViString s_multiMkrOFF[] = {"MLF1","MLF2","MLF3","MLF4","MLF5","MLF6","MLF7","MLF8","MLF9","MLF10"};
static ViString s_multiMkrSearch[] = {"MLSF?","MLSL?"};
static ViString s_readmultiMkrStatus[] = {"MLT?"};
static ViString s_multiMkrMessage[] = {"multimarkers inactive","multimarkers active"}; 
static ViString s_exeXdBDown[] = {"XDB", "XDL", "XDR"};
static ViString s_exeAMMeas[] = {"AMMOD OFF", "AMMOD"};

/* Annotation */
static ViString s_annotation[] = {"ANNOT OFF", "ANNOT ON"};

/* Set Mixer Parameters */
static ViString s_bandLock[] = {"BNDLC OFF", "BNDLC ON"};
static ViString s_lossFreq[] = {"LVF OFF", "LVF ON"};

/* Peak Search */                                
static ViString s_continuousSearch[] = {"CPF","CPN","CP OFF","CP ON"};
static ViString s_signalTrack[] = {"SGF","SGN","SG OFF","SG ON"};
static ViString s_peakRange[] = {"PSN","PSU","PSL","DY"};
static ViString s_readPeakOptions[] = {"CP?","SG?","DY?"};  
static ViString s_peakSearch[] = {"PS","NXP","NXL","NXR","MIS","MMS"};

/* Marker --> */
static ViString s_deltaMarkerTo[] = {"MTCF","MTSP","MTCS","MTMKS"};
static ViString s_normalMarkerTo[] = {"MKCF","MKRL","MKCS","MKMKS"};
static ViString s_peakTo[] = {"PKRL","PKCF"};
static ViString s_peakList[] = {"PLS FREQ", "PLS LEVEL ", "PLS OFF"};

/* Gate parameters */
static ViString s_gateSlope[] = {"GTSLP FALL", "GTSLP RISE"};
static ViString s_gateExtIn[] = {"GEX OFF", "GEX ON"};

/* Measurement window */
static ViString s_positionWindow[]  = {"WLX","WDX"};
static ViString s_showWindow[]  = {"WDO OFF","WDO ON"};
static ViString s_readMeasWindow[] = {"WDO?","WLX?","WDX?"};

/* Display Line and Reference Line */
static ViString s_showDL[] = {"DLF","DLN","DL OFF","DL ON"};
static ViString s_readDL[] = {"DLON?","DL?"};
static ViString s_showRefLine[] = {"RLF","RLN","RLN OFF","RLN ON"};
static ViString s_readRefLine[] = {"RLON?","RLN?","RLNON?","RLN?"};

/* Limit Lines */
static ViString s_LLType[] = {"LIMTYP TIME","LIMTYP FREQ"};
static ViString s_deleteLL[] = {"LMTBDEL","LMTADEL"};
static ViString s_setLL[] = {"LMTBIN","LMTAIN"};
static ViString s_showLL1[] = {"LMTA OFF","LMTA ON"};
static ViString s_showLL2[] = {"LMTB OFF","LMTB ON"}; 
static ViString s_positionX[] = {"LIMPOS ABS","LIMPOS REL","LIMPOS CENT"};
static ViString s_positionY[] = {"LIMAPOS ABS","LIMAPOS REL","LIMAPOS BOTM"};
static ViString s_shiftLL[] = {"LIMASFT","LIMSFT"};
static ViString s_LLpassfail[] = {"PFC OFF","PFC ON"};
static ViString s_LLjudgement[] = {"OPF?", "FPU?", "FPL?"};
static ViString s_judgementText[] = {"PASS", "UPPER FAIL", "LOWER FAIL", "UPPER&LOWER FAIL", "ERROR"};
static ViString s_readLLstatus[] = {"LMTB?","LMTA?"}; 
static ViString s_readLLoptions[] = {"LIMPOS?","LIMAPOS?"};
static ViString s_readLLmessage[] = {"Absolute","Relative Left","Relative Center","Absolute","Relative Top","Relative Bottom"};

/* Limit UP and LOW - level window */
static ViString s_showLimit[] = {"PFF","PFN","WLL","WUL"};
static ViString s_readLimit[] = {"PFON?","WLL?","WUL?","PCM?"};
static ViString s_judgmentText[] = {"FAIL","PASS"};

/* calibration and auto tuning */
static ViString s_calLevel[] = {"CLN","CLF"};                                  
static ViString s_calibration[]  = {"CLA","CLG","IT1","IT2","IT3","IT4","IT6",
									"CLALL","CLGAIN","CLSTEP","CLRBW","CLLOG","CLMAG","CLPBW"};
static ViString s_freqCorrection[] = {"FC OFF","FC ON"};
static ViString s_calCorrection[]  = {"CC OFF","CC ON"};
static ViString s_label[] = {"LOF","LON/"};
static ViString s_clearStatusByte[] = {"*CLS"};
static ViString s_autotune[] = {"TN"};

/* power measurement */
static ViString s_powerMeas[] = {"PWCH", "PWTOTAL", "PWAVG", "PWCARR", "PWM"};
static ViString s_readPowerMeas[] = {"PWCH?", "PWTOTAL?", "PWAVG?", "PWCARR?"};

/* ACP Measurement */
static ViString s_rootNyqFilter[] = {"NQST OFF", "NQST ON"};
static ViString s_BSWindow[] = {"ACPBSW OFF", "ACPBSW ON"};
static ViString s_ACPScreen[] = {"ACPSCR FULL", "ACPSCR SEPA", "ACPSCR CARR"};

/* OBW Measurement */
static ViString s_exeOBW[] = {"OBW OFF", "OBW"};

/* Spectrum mask Measurement */
static ViString s_dispMdSpecMask[] = {"SPMMOD ABS", "SPMMOD REL"};
static ViString s_exeSpecMask[] = {"SPM OFF", "SPM"};

/* Spurious Measurement */
static ViString s_passJudge[] = {"SPRJ LOW", "SPRJ UP"};

/* data management */
static ViString s_saveItemSetup[] = {"SVSET OFF", "SVSET ON"};
static ViString s_saveItemTrcData[] = {"SVTRC OFF", "SVTRC ON"};
static ViString s_saveItemAntenna[] = {"SVANT OFF", "SVANT ON"};
static ViString s_saveItemNormCorr[] = {"SVNRM OFF", "SVNRM ON"};
static ViString s_saveItemTrcLevel[] = {"SVLVL OFF", "SVLVL ON"};
static ViString s_saveItemLmtLines[] = {"SVLIM OFF", "SVLIM 1", "SVLIM 2", "SVLIM3"};
static ViString s_saveItemLossFreq[] = {"SVLOSS OFF", "SVLOSS ON"};
static ViString s_saveItemChannel[] = {"SVCH OFF", "SVCH ON"};
static ViString s_saveItemSpurious[] = {"SVSPR OFF", "SVSPR ON"};
static ViString s_printerR3131[] = {"PRTCMD ESC", "PRTCMD PCL", "PRTCMD ESCPR"};
static ViString s_prtModeR3131[] = {"HCIMAG MON", "HCIMAG GRY"};
static ViString s_printSize[] = {"PSIZE LRG", "PSIZE SML"};
static ViString s_printerR3132s[] = {"PRTCMD ESC", "PRTCMD PCL", "PRTCMD ESCR"};
static ViString s_prtModeR3132s[] = {"PRT COL", "PRT SCOLS", "PRT SCOLL",
									 "PRT GRY", "PRT MOS", "PRT MOL"};
static ViString s_BMPMode[] = {"HCIMAG COL", "HCIMAG SCOL", "HCIMAG GRY", "HCIMAG MON"};
static ViString s_fileCompress[] = {"HCCMPRS OFF", "HCCMPRS ON"};

/* antenna factor correction */                                  
static ViString s_showAC[] = {"ANT OFF","ANT4"};
static ViString s_workAC[] = {"CRDEL","CRIN"};
static ViString s_modeAC[] = {"CR LVL","CR ANT"};
static ViString s_antennaAC[] = {"ANT0","ANT1","ANT2","ANT3","ANT OFF"};
static ViString s_readAC[] = {"ANT?","CR?","no antenna correction","dipole antenna correction",
                              "log-periodic antenna correction","biconical antenna correction",
                              "Bilog antenna correction", "user antenna correction", "user level correction"};                                   

/* tracking generator */
static ViString s_setTGstatus[] = {"TGF", "TG", "TGL"};
static ViString s_readTGstatus[] = {"TG?", "TGL?"};
static ViString s_TGfreqComp[] = {"TGM", "TGA","TGM?"}; 
static ViString s_TGnormFunction[] = {"NORM ON", "NORM OFF", "NORM EX","NORM?"};

/* Multi Screen */
static ViString s_multiScreenMode[] = {"MLTSCR ZM", "MLTSCR FT", "MLTSCR TT", "MLTSCR OFF"};
static ViString s_screenSide[] = {"SCRSEL TRB", "SCRSEL TRA"};

/* EMC */
static ViString s_traceDetect[] = {"EMCDET QP", "EMCDET PEAK", "EMCDET NRM"};
static ViString s_QPBandWidth[] = {"QA", "QP0", "QP1", "QP2", "QP3"};
static ViString s_EMCAntenna[] = {"AN0", "AN1", "AN2", "AN3", "AN4", "AF"};

/* Sound */
static ViString s_soundMode[] = {"SON", "SAM", "SFM", "SOF"};
static ViString s_squelchMode[] = {"SQE OFF", "SQE ON"};

/* Noise/Hz */
static ViString s_noiseHzMode[] = {"NIM", "NIU", "NIC", "NIF"};

/* utility */
static ViString s_selfTest[] = {"*TST?"};
static ViString s_selfTestMessage[] = {"CPU","A/D","Lock Detector","Total Gain","Cal Signal",
									   "RF ATT","Variable Gain AMP","IF step AMP","RBW SWP"};
static ViString s_errorQuery[] = {"ERRNO?"};
static ViString s_freqUnit[] = {"Hz","kHz","MHz","GHz"};
static ViString s_timeUnit[] = {"ns","us","ms","s"};
static ViString s_levelUnit[] = {"DB","DB","DB","","","V","W"};
static ViString s_positionUnit[] = {"SC","HZ"};
static ViString s_textONOFF[] = {"inactive","active"};

/*****************************************************************************/
/* INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE                               */
/*****************************************************************************/

struct avr3132s_stringValPair {
    ViStatus stringVal;
    ViString stringName;
};

struct avr3132s_instrFeatures {
    ViReal64 centerLow;
    ViReal64 centerHigh;
    ViReal64 startLow;
    ViReal64 startHigh;
    ViReal64 stopLow;
    ViReal64 stopHigh;
    ViReal64 freqOffsetMin;
    ViReal64 freqOffsetMax;
    ViReal64 spanLow;
    ViReal64 spanHigh;
    ViReal64 inputAttMin;
    ViReal64 inputAttMax;
    ViReal64 refLevelOffsetMin;
    ViReal64 refLevelOffsetMax;
    ViReal64 sweepTimeMini;
    ViReal64 sweepTimeMaxi;
    ViInt16  averageCountMini;
    ViInt16  averageCountMaxi;
    ViReal64 DLlevelMin;
    ViReal64 DLlevelMax;
    ViInt16  refUnit;
    ViInt16  span;
    ViUInt32 instrumentType;
    ViReal64 deltaYmin;
    ViReal64 deltaYmax;
    ViReal64 calLevelMin;
    ViReal64 calLevelMax;
    ViChar   instrDriverRevision[256];
    ViInt16  MKRMstatus[10]; /*0 if multi-marker inactive, 1 if muti-marker active*/   
};

typedef struct avr3132s_stringValPair avr3132s_tStringValPair;
    
typedef struct avr3132s_instrFeatures *avr3132s_ptrFeatures;

/*****************************************************************************/
/* UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions)                   */
/*****************************************************************************/

ViBoolean avr3132s_invalidViBooleanRange (ViBoolean val);
ViBoolean avr3132s_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean avr3132s_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean avr3132s_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max);
ViBoolean avr3132s_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean avr3132s_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max);
ViBoolean avr3132s_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus avr3132s_initCleanUp (ViSession openRMSession,ViPSession openInstrSession,
                           ViStatus currentStatus);
ViStatus avr3132s_defaultInstrSetup (ViSession instrSession, ViUInt32 instrType);
ViStatus avr3132s_initReset (ViSession, ViUInt32);
ViStatus avr3132s_Search_Scale (ViSession instrSession, ViPInt16 scale);
ViStatus avr3132s_Search_RefOffsetStatus (ViSession instrSession, ViPInt16 refOffsetStatus);

/*****************************************************************************/
/* USER-CALLABLE FUNCTIONS (Exportable Functions)                            */
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_init (ViRsrc resourceName, ViBoolean IDquery,
                             ViBoolean reset, ViPSession instrSession)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViSession rmSession = 0;
    ViUInt32 retCnt = 0;
    ViChar Buffer[BUFFER_SIZE];
    ViChar interim[5];
    ViUInt32 avr3132s_instrType,retCount;
    ViByte cable;
    
    /*- Check input parameter ranges ----------------------------------------*/

    if (avr3132s_invalidViBooleanRange (IDquery))
        return VI_ERROR_PARAMETER2;
        
    if (avr3132s_invalidViBooleanRange (reset))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/

    if ((avr3132s_status = viOpenDefaultRM (&rmSession)) < 0)
        return avr3132s_status;

    if ((avr3132s_status = viOpen (rmSession, resourceName, VI_NULL, VI_NULL, instrSession)) < 0) {
        viClose (rmSession);
        return avr3132s_status;
    }

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    
    cable = resourceName[0]; /* GPIB or RS-232 cable */
    
    if (cable=='G') { /*GPIB cable*/

        if ((avr3132s_status = viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 10000)) < 0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
            
        if ((avr3132s_status = viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000)) < 0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
            
        if ((avr3132s_status = viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                                           VI_FLUSH_ON_ACCESS))   < 0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
            
        if ((avr3132s_status = viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                                           VI_FLUSH_ON_ACCESS))   < 0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
        
        Fmt (Buffer,"%s<%s",s_init[0]);     
        if ((avr3132s_status = viWrite(*instrSession,Buffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
            
    }
    
    else { /*RS-232 cable*/
        /* Serial Interface initialized to 8 databits, 1 stopbit, no parity, */
        /* baud rate = 19200, flow control = XonXoff and timeout = 20000 ms  */
        
        if ((avr3132s_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_DATA_BITS, 8))<0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
    
        if ((avr3132s_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_STOP_BITS,VI_ASRL_STOP_ONE))<0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
            
        if ((avr3132s_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_PARITY, VI_ASRL_PAR_NONE))<0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
        
        if ((avr3132s_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_BAUD, 19200))<0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
        
        if ((avr3132s_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_FLOW_CNTRL,VI_ASRL_FLOW_XON_XOFF))<0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
        
        if ((avr3132s_status = viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 20000)) < 0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
   
        if ((avr3132s_status = viSetBuf (*instrSession, VI_ASRL_IN_BUF|VI_ASRL_OUT_BUF, 512)) < 0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
   
        if ((avr3132s_status = viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000)) < 0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
   
        if ((avr3132s_status = viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,VI_FLUSH_ON_ACCESS))<0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
    
        if ((avr3132s_status = viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,VI_FLUSH_ON_ACCESS)) < 0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
 
        if ((avr3132s_status = viSetAttribute (*instrSession, VI_ATTR_TERMCHAR, '\n')) < 0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
 
        if ((avr3132s_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_END_IN,VI_ASRL_END_TERMCHAR)) < 0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
   
        if ((avr3132s_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_END_OUT,VI_ASRL_END_NONE)) < 0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);       

      // sends an empty string to the analyzer to init the RS-232 link //
        Fmt (Buffer,"%s<%s\r\n",s_init[2]);    
        if ((avr3132s_status = viWrite(*instrSession,Buffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
            
    }       

    /*- Identification Query -------------------------------------------------*/
    
    if (IDquery) {  /* identification executed */
    
        Fmt (Buffer,"%s<%s\r\n",s_init[1]);     
        if ((avr3132s_status = viWrite(*instrSession,Buffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
    
        if ((avr3132s_status = viRead (*instrSession, Buffer, BUFFER_SIZE, &retCnt)) < 0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
            
        
        if (Buffer[10] != 'R')  
            return avr3132s_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
         
        interim[0] = Buffer[11];
        interim[1] = Buffer[12];
        interim[2] = Buffer[13];
        interim[3] = Buffer[14];
        interim[4] = Buffer[(ViInt16)(retCount)-2];
     
        if (Scan(interim, "%s>%d[b4]", &avr3132s_instrType)!=1) {
            avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
        }
    }
    else 
        avr3132s_instrType = 3131 ;  /* no identification executed */
        
            
    /*- Reset instrument ----------------------------------------------------*/
    if (reset) {
        if ((avr3132s_status = avr3132s_initReset (*instrSession,avr3132s_instrType)) < 0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
    }      
    else { /*- Send Default Instrument Setup ---------------------------------*/ 
        if ((avr3132s_status = avr3132s_defaultInstrSetup (*instrSession,avr3132s_instrType)) < 0)
            return avr3132s_initCleanUp (rmSession, instrSession, avr3132s_status);
    }
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function : configure and read                                             */
/* Purpose : application function                                            */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_configRead(ViSession instrID,ViBoolean trace,ViInt16 peakSearch,
                                  ViInt16 _VI_FAR traceArray[],
                                  ViPReal64 sweepTime, ViChar _VI_FAR sweepUnit[],
                                  ViPReal64 MKRNposition,ViPReal64 MKRNlevel)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    avr3132s_ptrFeatures featuresPtr;
    ViInt16 status,control;
    ViReal64 deltaposition, deltalevel, sweepValue;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
       
    if (avr3132s_invalidViBooleanRange (trace))             
        return VI_ERROR_PARAMETER2;
        
    if (avr3132s_invalidViInt16Range (peakSearch,0,5))             
        return VI_ERROR_PARAMETER3;
        
    if ((avr3132s_status = avr3132s_Read_TraceData (instrID, trace, 0, traceArray))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = avr3132s_Search_Peak (instrID, peakSearch, &deltaposition, &deltalevel)) <0)
        return avr3132s_status;   
       
    if ((avr3132s_status = avr3132s_Read_SweepTime (instrID, &sweepValue, &control))<0)
        return avr3132s_status;
       
    if ((avr3132s_status = avr3132s_Convert_Time (instrID, sweepValue, sweepTime, sweepUnit))<0)
        return avr3132s_status;
       
    if ((avr3132s_status = avr3132s_Read_NormalMarker (instrID, MKRNposition, MKRNlevel))<0)
        return avr3132s_status;   
       
    return avr3132s_status;
} 

/*===========================================================================*/
/* Function: Center Frequency	                                             */           
/* Purpose:  sets the center frequency.                                      */           
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_CenterFrequency (ViSession instrID,
                                            ViReal64 cntrFreq,
                                            ViBoolean stepCF, ViReal64 stepSizeCF)
{
	ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount = 0;
    ViUInt32 total = 0;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    if (avr3132s_invalidViReal64Range (cntrFreq, featuresPtr->centerLow, featuresPtr->centerHigh))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViBooleanRange (stepCF))
    	return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViReal64Range (stepSizeCF, featuresPtr->centerLow, featuresPtr->centerHigh))             
        return VI_ERROR_PARAMETER4;
        
    Fmt (wrtBuffer, "%s<CF %fHZ", cntrFreq);
    total += NumFmtdBytes();
    
    if (stepCF==VI_ON)
    	Fmt (wrtBuffer, "%s[a]<CA");
    else
    	Fmt (wrtBuffer, "%s[a]<CS %fHZ", stepSizeCF);
    total += NumFmtdBytes();	
    
    if ((avr3132s_status = viWrite(instrID, wrtBuffer, total, &retCount)) < 0)
    	return avr3132s_status;
    
    return avr3132s_status;	
}

/*===========================================================================*/
/* Function: Frequency Span		                                             */           
/* Purpose:  sets the frequency span. 		                                 */           
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_Span (ViSession instrID, ViInt16 spanType,
                                                    ViReal64 spanValue)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    if (avr3132s_invalidViInt16Range (spanType, 0, 4))             
        return VI_ERROR_PARAMETER2;                         
       
    if (spanType==0) {  /* Manual Span Setting */ 
        if (avr3132s_invalidViReal64Range (spanValue, featuresPtr->spanLow, featuresPtr->spanHigh))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s %fHZ",s_spanType[spanType],spanValue);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;
    }
    else { /*full span,zero span, Peak Zoom or last span*/
        Fmt (wrtBuffer,"%s<%s",s_spanType[spanType]);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;
    }
    
    if ((spanType==2)||(spanValue==0)) featuresPtr->span=0;  /*span null*/
    else featuresPtr->span=1; /*span not null*/
     
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Start and Stop Frequencies                                      */           
/* Purpose:  sets start and stop frequency.					                 */           
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_StartStop (ViSession instrID,
                                      ViReal64 strFreq, ViReal64 stpFreq)
{
	ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount = 0;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    if (avr3132s_invalidViReal64Range (strFreq, featuresPtr->startLow, featuresPtr->startHigh))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViReal64Range (stpFreq, featuresPtr->stopLow, featuresPtr->stopHigh))             
        return VI_ERROR_PARAMETER3;
        
    Fmt (wrtBuffer, "%s<FA %fHZ; FB %f HZ", strFreq, stpFreq);
    
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avr3132s_status;
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set Frequencies                                                 */           
/* Purpose:  sets up center, span, start or stop frequency.                  */           
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_Freq (ViSession instrID, ViInt16 freqType,
                                 ViReal64 cntr_str, ViReal64 span_stp)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    ViReal64 span;

    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    if (avr3132s_invalidViInt16Range (freqType, 0, 5))             
        return VI_ERROR_PARAMETER2;
        
    switch (freqType)
    {
    case 0:   // Center
    	if (avr3132s_invalidViReal64Range (cntr_str, featuresPtr->centerLow, featuresPtr->centerHigh))             
        	return VI_ERROR_PARAMETER3;
    	Fmt (wrtBuffer,"%s<CF %fHZ", cntr_str);
    	break;
    	
    case 1:   // Span
    	if (avr3132s_invalidViReal64Range (span_stp, featuresPtr->spanLow, featuresPtr->spanHigh))             
        	return VI_ERROR_PARAMETER4;
    	Fmt (wrtBuffer,"%s<SP %fHZ", span_stp);
    	break;
    	
    case 2:   // Center and Span
    	if (avr3132s_invalidViReal64Range (cntr_str, featuresPtr->centerLow, featuresPtr->centerHigh))             
        	return VI_ERROR_PARAMETER3;
    	if (avr3132s_invalidViReal64Range (span_stp, featuresPtr->spanLow, featuresPtr->spanHigh))             
        	return VI_ERROR_PARAMETER4;
    	Fmt (wrtBuffer,"%s<CF %fHZ; SP %fHZ", cntr_str, span_stp);
    	break;
    	
    case 3:   // Start
    	if (avr3132s_invalidViReal64Range (cntr_str, featuresPtr->startLow, featuresPtr->startHigh))             
        	return VI_ERROR_PARAMETER3;
    	Fmt (wrtBuffer,"%s<FA %fHZ", cntr_str);
    	break;
    	
    case 4:   // Stop
    	if (avr3132s_invalidViReal64Range (span_stp, featuresPtr->stopLow, featuresPtr->stopHigh))             
        	return VI_ERROR_PARAMETER4;
    	Fmt (wrtBuffer,"%s<FB %fHZ", span_stp);
    	break;
    	
    case 5:   // Start and Stop
    	if (avr3132s_invalidViReal64Range (cntr_str, featuresPtr->startLow, featuresPtr->startHigh))             
        	return VI_ERROR_PARAMETER3;
    	if (avr3132s_invalidViReal64Range (span_stp, featuresPtr->stopLow, featuresPtr->stopHigh))             
        	return VI_ERROR_PARAMETER4;
    	Fmt (wrtBuffer,"%s<FA %fHZ; FB %fHZ", cntr_str, span_stp);
    	break;	
    }
    
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
        
    // checks whether the span is null or not 
    avr3132s_Read_FreqSettings (instrID, 1, &span);
    if (span==0) featuresPtr->span = 0;
    else featuresPtr->span = 1;    

    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Trace Math				                                         */
/* Purpose:  This function executes trace math operations.                   */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Execute_TraceMath (ViSession instrID, ViInt16 oprMode)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (avr3132s_invalidViInt16Range (oprMode, 0, 4))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer, "%s<%s", s_traceMath[oprMode]);
    
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
        
    return avr3132s_status;    
}

/*===========================================================================*/
/* Function: Set Multi Screen		                                         */
/* Purpose:  This function sets multi-screen display conditions.             */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_MultiScreen (ViSession instrID,
                                        ViInt16 screenMode, ViReal64 position,
                                        ViReal64 width, ViBoolean screenSide)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount = 0;
    ViUInt32 total = 0;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    if (avr3132s_invalidViInt16Range (screenMode, 0, 3))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViReal64Range (position, featuresPtr->centerLow, featuresPtr->centerHigh))             
        return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViReal64Range (width, featuresPtr->centerLow, featuresPtr->centerHigh))             
        return VI_ERROR_PARAMETER4;
    if (avr3132s_invalidViBooleanRange (screenSide))             
    	return VI_ERROR_PARAMETER5;
    
    Fmt (wrtBuffer, "%s<%s", s_multiScreenMode[screenMode]);
    total += NumFmtdBytes();
    
    if (screenMode==0)
    {
    	Fmt (wrtBuffer, "%s[a]<; ZMPOS %fHZ; ZMWID %fHZ", position, width);
    	total += NumFmtdBytes();
    }
    
    else if (screenMode==1)
    {
    	Fmt (wrtBuffer, "%s[a]<; ZMPOS %fHZ", position);
    	total += NumFmtdBytes();
    }
    
    if (screenMode<=2)
    {
    	Fmt (wrtBuffer, "%s[a]<; %s", s_screenSide[screenSide]);
    	total += NumFmtdBytes();
    }
    	 
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,total,&retCount)) < 0)
        return avr3132s_status;
        
    return avr3132s_status;	 
}

/*===========================================================================*/
/* Function: Set EMC				                                         */
/* Purpose:  This function sets the EMC measurement parameters.              */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_EMC (ViSession instrID, ViInt16 traceDetect,
								ViInt16 bandWidth, ViInt16 antenna)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViInt16Range (traceDetect, 0, 2))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViInt16Range (bandWidth, 0, 4))             
        return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViInt16Range (antenna, 0, 5))             
        return VI_ERROR_PARAMETER4;
        
    Fmt (wrtBuffer, "%s<%s; %s; %s", s_traceDetect[traceDetect], 
    	 s_QPBandWidth[bandWidth], s_EMCAntenna[antenna]);
    	 
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
        
    return avr3132s_status;	 
}

/*===========================================================================*/
/* Function: Read  Frequency Settings                                        */
/* Purpose:  Reads a frequency that you have chosen                          */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_FreqSettings (ViSession instrID, ViInt16 parameter,
                                                             ViPReal64 value)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViInt16Range (parameter,0,3))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (Buffer,"%s<%s\r\n",s_readFreq[parameter]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
       
    if (Scan(Buffer,"%s>%f",value)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set Frequency Offset                                            */           
/* Purpose:  sets up frequency offset and activates frequency offset         */           
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_FreqOffset (ViSession instrID, ViBoolean activation,
                                                          ViReal64 freqOffset)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
       
    if (avr3132s_invalidViBooleanRange (activation))             
        return VI_ERROR_PARAMETER2;   

    if (activation == VI_ON) { /*activation*/
        if (avr3132s_invalidViReal64Range (freqOffset,featuresPtr->freqOffsetMin,
                                                  featuresPtr->freqOffsetMax))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s%fHZ\r\n", s_freqOffset[activation], freqOffset);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;
    }
    else { /*desactivation*/
        Fmt (wrtBuffer,"%s<%s\r\n",s_freqOffset[activation]);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;
    }
     
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read Frequency Offset                                           */
/* Purpose:  Reads the frequency offset status and value                     */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_FreqOffset (ViSession instrID, ViPInt16 status,
                                                           ViPReal64 value)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    // returns the frequency offset status
    
    Fmt (Buffer,"%s<%s\r\n",s_readFreqOffset[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
       
    if (Scan(Buffer,"%s>%d[b2]",status)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    // returns the frequency offset value
    
    Fmt (Buffer,"%s<%s\r\n",s_readFreqOffset[1]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
       
    if (Scan(Buffer,"%s>%f",value)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
     
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set Frequency Step Size                                         */           
/* Purpose:  sets up and activates or deactivates Frequency Step Size        */           
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_FreqStepSize (ViSession instrID, ViBoolean controlType,
                                                            ViReal64 freqStepSize)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;

    if (avr3132s_invalidViBooleanRange (controlType))             
        return VI_ERROR_PARAMETER2;
       
    if (controlType == VI_ON) { /*step size chosen by the analyzer*/ 
        Fmt (wrtBuffer,"%s<%s\r\n", s_freqStepSize[controlType]);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;
    }
    else { /*step size chosen by the user*/   
        if (avr3132s_invalidViReal64Range (freqStepSize,featuresPtr->centerLow,
                                                    featuresPtr->centerHigh))             
            return VI_ERROR_PARAMETER3;   
        Fmt (wrtBuffer,"%s<%s%fHZ\r\n",s_freqStepSize[controlType],freqStepSize);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;
    }
     
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read Frequency Step Size                                        */
/* Purpose:  returns the frequency step size and says whether this parameter */
/*           is controled  by the user or by the analyzer                    */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_FreqStepSize (ViSession instrID, ViPReal64 value,
                                                             ViPInt16 controlType)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    /* returns the value of the center frequency step size */
    Fmt (Buffer,"%s<%s\r\n",s_readFreqStepSize[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f",value)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    /* returns the type of control (auto/user) for this parameter */
    Fmt (Buffer,"%s<%s\r\n",s_readFreqStepSize[1]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", controlType)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
     
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set Frequency Counter                                           */
/* Purpose:  Sets the counter ON/OFF or sets the counter resolution          */
/*===========================================================================*/                               
ViStatus _VI_FUNC avr3132s_Set_FreqCounter (ViSession instrID, ViBoolean activation,
                                                           ViInt16 resolution)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViBooleanRange (activation))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViInt16Range (resolution,0,3))             
        return VI_ERROR_PARAMETER3;
       
    if (activation == VI_ON) {
        Fmt (wrtBuffer,"%s<%s\r\n", s_counterResolution[resolution]);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status; 
    }

    Fmt (wrtBuffer,"%s<%s\r\n", s_counterONOFF[activation]);     
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;

    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read Frequency Counter                                          */
/* Purpose:  returns the frequency counter value and says whether the counter*/
/*           is active or inactive                                           */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_FreqCounter (ViSession instrID,ViPInt16 counterStatus,
                                                           ViPInt16 counterResolution,
                                                           ViPReal64 counterValue)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    avr3132s_ptrFeatures featuresPtr;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    
    /* returns the frequency counter status */
    
    Fmt (Buffer,"%s<%s\r\n",s_readFreqCounter[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
        
    if (Scan(Buffer,"%s>%d[b2]", counterStatus)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    /* returns the frequency read by the counter */
    
    Fmt (Buffer,"%s<%s\r\n",s_readFreqCounter[1]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
        
    if (Scan(Buffer,"%s>%f", counterValue)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    /* returns the frequency counter resolution */
    
    if (featuresPtr->instrumentType==3131) {
    	Fmt (Buffer,"%s<%s\r\n",s_readFreqCounter[2]);   
    	if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        	return avr3132s_status;
    }
    else {
    	Fmt (Buffer,"%s<%s\r\n",s_readFreqCounter[3]);   
    	if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        	return avr3132s_status;
    }    	
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
        
    if (Scan(Buffer,"%s>%d[b2]", counterResolution)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }

    return avr3132s_status;
}
    
/*===========================================================================*/
/* Function: Frequency Preselector                                           */           
/* Purpose:  This function adjusts a tracking between the sweep frequency of */
/*			 the preselector embedded in the input part and the sweep input  */
/*			 frequency in the input frequency range of 3.0 to 8.0 GHz.       */                   	            
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_FreqPreselector (ViSession instrID,
                                            ViBoolean peaking, ViInt16 peakingValue)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    avr3132s_ptrFeatures featuresPtr;
    
    if (avr3132s_invalidViBooleanRange (peaking))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViInt16Range (peakingValue, -127, 127))             
        return VI_ERROR_PARAMETER3;
        
    if (peaking==VI_ON)
    	Fmt (Buffer, "%s<PPA");
    else
    	Fmt (Buffer, "%s<PPM %d", peakingValue);
    
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
    
    return avr3132s_status;	
}

/*===========================================================================*/
/* Function: Hi Sense			                                             */           
/* Purpose:  This function switches build-in pre-amplifier at the gain of    */
/*			 20 dB and over.					                         	 */           
/*===========================================================================*/ 
ViStatus _VI_FUNC avr3132s_set_HiSense (ViSession instrID, ViBoolean hiSense)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    avr3132s_ptrFeatures featuresPtr;
    
    if (avr3132s_invalidViBooleanRange (hiSense))             
        return VI_ERROR_PARAMETER2; 
        
    Fmt (Buffer, "%s<%s", s_hiSense[hiSense]);
    
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Input Impedance	                                             */           
/* Purpose:  This function sets the impedance input level for the display. 	 */           
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_InputImpedance (ViSession instrID, ViBoolean impedance)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    avr3132s_ptrFeatures featuresPtr;
    
    if (avr3132s_invalidViBooleanRange (impedance))             
        return VI_ERROR_PARAMETER2; 
        
    Fmt (Buffer, "%s<%s", s_impedance[impedance]);
    
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
    
    return avr3132s_status;    
}

/*===========================================================================*/
/* Function: Set Carrier Channel                                             */           
/* Purpose:  This function sets center frequency by channel number. This 	 */
/*			 function is effective, when Input Type is set to CH Type 1.     */           
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_CarrierChannel (ViSession instrID, ViInt16 crrChnl)
{
	ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViInt16Range (crrChnl, -9999, 9999))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (Buffer, "%s<CH %d", crrChnl);    
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
    
    return avr3132s_status;    
}

/*===========================================================================*/
/* Function: Set Center Channel                                              */           
/* Purpose:  This function sets center frequency by channel number. This 	 */
/*			 function is effective, when Input Type is set to CH Type 2.     */           
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_CenterChannel (ViSession instrID, ViInt16 cntrChnl)
{
	ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViInt16Range (cntrChnl, 0, 9999))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (Buffer, "%s<CFCH %d", cntrChnl);    
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
    
    return avr3132s_status;     
}

/*===========================================================================*/
/* Function: Set Start and Stop Channels                                     */           
/* Purpose:  This function sets start and stop frequencies by channel 		 */
/*			 numbers. This function is effective, when Input Type is set to  */
/*			 CH Type 2.     												 */           
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_StartStopChannel (ViSession instrID,
                                                 ViInt16 strChnl, ViInt16 stpChnl)
{
	ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViInt16Range (strChnl, 0, 9999))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViInt16Range (stpChnl, 0, 9999))             
        return VI_ERROR_PARAMETER3;
        
    Fmt (Buffer, "%s<FACH %d; FBCH %d", strChnl, stpChnl);    
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
    
    return avr3132s_status;     
}

/*===========================================================================*/
/* Function: Set Frequency Input		                                     */           
/* Purpose:  This function selects frequency input method by frequency 		 */
/*			 directly, CH Type 1 table or CH Type 2 Table.					 */           
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_FrequencyInput (ViSession instrID, ViInt16 inputType,
                                               ViReal64 strCHShft, ViReal64 stpCHShft)
{
	ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViInt16Range (inputType, 0, 2))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViReal64Range (strCHShft, 0.0, 1.0e9))             
        return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViReal64Range (stpCHShft, 0.0, 1.0e9))             
        return VI_ERROR_PARAMETER4;
        
    Fmt (Buffer, "%s<%s; FACHO %fHZ; FBCHO %fHZ", 
    	 s_inputType[inputType], strCHShft, stpCHShft);
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
    
    return avr3132s_status;	 
}

/*===========================================================================*/
/* Function: Edit CH Type 1				                                     */           
/* Purpose:  This function edits the tables of the channel type 1.			 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Edit_CHType1 (ViSession instrID, ViInt16 editTable,
                                         ViInt16 strChnl, ViInt16 stpChnl,
                                         ViReal64 strFreq, ViReal64 chnlSpc,
                                         ViInt16 chnlOffset)
{
	ViStatus avr3132s_status = VI_SUCCESS;
    avr3132s_ptrFeatures featuresPtr;      
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
       
    if (avr3132s_invalidViInt16Range (editTable, 0, 2))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViInt16Range (strChnl, -9999, 9999))             
        return VI_ERROR_PARAMETER3; 
    if (avr3132s_invalidViInt16Range (stpChnl, -9999, 9999))             
        return VI_ERROR_PARAMETER4; 
    if (avr3132s_invalidViReal64Range (strFreq, 0.0, featuresPtr->startHigh))             
        return VI_ERROR_PARAMETER5; 
    if (avr3132s_invalidViReal64Range (chnlSpc, -100.0e6, 100.0e6))             
        return VI_ERROR_PARAMETER6; 
    if (avr3132s_invalidViInt16Range (chnlOffset, -9999, 9999))             
        return VI_ERROR_PARAMETER7;
        
    Fmt (Buffer, "%s<%s %d,%d,%fHZ,%fHZ,%d", 
    	 s_editTable[editTable], strChnl, stpChnl, strFreq, chnlSpc, chnlOffset);
	if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
    
    return avr3132s_status;	 
}

/*===========================================================================*/
/* Function: Set CH Type 1				                                     */           
/* Purpose:  This function sets enable or disable for the tables of the		 */
/*			 channel type 1.			 									 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_CHType1 (ViSession instrID, ViBoolean table1,
										ViBoolean table2, ViBoolean table3)
{
	ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViBooleanRange (table1))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViBooleanRange (table2))             
        return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViBooleanRange (table3))             
        return VI_ERROR_PARAMETER4;
        
    Fmt (Buffer, "%s<CHTBL1 %s; CHTBL2 %s; CHTBL3 %s",
    	 s_EnabeDisable[table1], s_EnabeDisable[table2], s_EnabeDisable[table3]);
	if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
    
    return avr3132s_status;	
}

/*===========================================================================*/
/* Function: Edit CH Type 2				                                     */           
/* Purpose:  This function edits the tables of the channel type 2.			 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Edit_CHType2 (ViSession instrID, ViInt16 tblSize,
                                         ViInt16 chnlNum[], ViReal64 crrFreq[],
                                         ViReal64 strFreq[], ViReal64 stpFreq[])
{
	ViStatus avr3132s_status = VI_SUCCESS;
    avr3132s_ptrFeatures featuresPtr;      
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViUInt32 total = 0;
    ViInt16  n, *pt_chnl;
    ViReal64 *pt_crr, *pt_str, *pt_stp;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
        
    if (avr3132s_invalidViInt16Range (tblSize, 0, 99))             
    	return VI_ERROR_PARAMETER2;
    
    n = tblSize;
    pt_chnl = chnlNum;
    pt_crr  = crrFreq;
    pt_str  = strFreq;
    pt_stp  = stpFreq;
    while (--n>=0)
    {
    	if (avr3132s_invalidViInt16Range (*pt_chnl++, 0, 9999))             
    		return VI_ERROR_PARAMETER3;
    	if (avr3132s_invalidViReal64Range (*pt_crr++, 0.0, featuresPtr->centerHigh))             
    		return VI_ERROR_PARAMETER4;
    	if (avr3132s_invalidViReal64Range (*pt_str++, 0.0, featuresPtr->startHigh))             
    		return VI_ERROR_PARAMETER5;
    	if (avr3132s_invalidViReal64Range (*pt_stp++, 0.0, featuresPtr->stopHigh))             
    		return VI_ERROR_PARAMETER6;	
    }
    
    Fmt (Buffer, "%s<CHEDDEL");
    total += NumFmtdBytes();
    
    n = tblSize;
    pt_chnl = chnlNum;
    pt_crr  = crrFreq;
    pt_str  = strFreq;
    pt_stp  = stpFreq;
    while (--n>=0)
    {
    	Fmt (Buffer, "%s[a]<; CHEDIN %d,%fHZ,%fHZ,%fHZ", 
    		 *pt_chnl++, *pt_crr++, *pt_str++, *pt_stp++);
    	total += NumFmtdBytes();	 
    }
    
    if ((avr3132s_status = viWrite(instrID, Buffer, total, &retCount)) < 0)
        return avr3132s_status;
    
    return avr3132s_status;	
}

/*===========================================================================*/
/* Function: Reference Level	                                             */           
/* Purpose:  sets the reference level (Y axis).                           	 */           
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_RefLevel (ViSession instrID, ViReal64 refLevel)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s%f",s_refLevel[0],refLevel);     
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read  Reference Level                                           */
/* Purpose:  returns the value (without unit) of the reference level         */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_RefLevel(ViSession instrID, ViPReal64 refLevelValue)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readRefLevel[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f", refLevelValue)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
     
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set Reference Level Offset                                      */           
/* Purpose:  Set up and activates the reference level offset.                */           
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_RefOffset (ViSession instrID, ViBoolean activation,
                                                         ViReal64 refLevelOffset)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
       
    if (avr3132s_invalidViBooleanRange (activation))             
        return VI_ERROR_PARAMETER2;   

    if (activation == VI_ON) {  /* activation */
        if (avr3132s_invalidViReal64Range (refLevelOffset,featuresPtr->refLevelOffsetMin,
                                                      featuresPtr->refLevelOffsetMax))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s%fDB\r\n", s_refOffset[2], refLevelOffset);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;
    }
    else { /* deactivation */
    	if (featuresPtr->instrumentType==3131) Fmt (wrtBuffer,"%s<%s\r\n",s_refOffset[0]);
    	else  Fmt (wrtBuffer,"%s<%s\r\n",s_refOffset[1]);
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;
    }
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read Reference Level Offset                                     */
/* Purpose:  returns the value (unit is always dB) of the reference level    */
/* offset and says whether this offset is active or not                      */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_RefLevelOffset (ViSession instrID,ViPReal64 value,
                                                              ViPInt16 activation)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
        
    Fmt (Buffer,"%s<%s\r\n",s_readRefOffset[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f",value)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    avr3132s_Search_RefOffsetStatus (instrID, activation); 
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set Reference Unit                                              */           
/* Purpose:  Set up the unit of the reference level.                         */           
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_RefUnit (ViSession instrID, ViInt16 refUnit)
{   
    ViStatus avr3132s_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
   
    if (avr3132s_invalidViInt16Range (refUnit,0,4))             
        return VI_ERROR_PARAMETER2;
    
    featuresPtr->refUnit = refUnit; 
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_refUnit[refUnit]);     
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
       
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read Reference Unit                                             */
/* Purpose:  returns the reference unit                                      */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_RefUnit (ViSession instrID,ViPInt16 unitNumber,
                                                       ViChar _VI_FAR unitName[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readRefUnit[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", unitNumber)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    Fmt(unitName,"%s<%s", s_readUnitName[*unitNumber]);  
    return avr3132s_status;
} 

/*===========================================================================*/
/* Function: Set Scale                                                       */           
/* Purpose:  Set up the scale of the Y axis .                                */           
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_Scale (ViSession instrID, ViInt16 scale)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    ViChar unitName[10];
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;

    if (avr3132s_invalidViInt16Range (scale,0,4))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_yScale[scale]);     
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
       
    // the reference unit may have been changed by the scale change
    avr3132s_Read_RefUnit (instrID, &featuresPtr->refUnit,unitName);
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read Scale                                                      */
/* Purpose:  reads the scale status number and text                          */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_Scale (ViSession instrID, ViPInt16 status,
                                                      ViChar _VI_FAR scaleName[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    avr3132s_ptrFeatures featuresPtr;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    
    avr3132s_Search_Scale (instrID, status);    
    Fmt(scaleName,"%s<%s", s_readScale[*status]);
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set Input Attenuation                                           */           
/* Purpose:  Set up the Input attenuation.                                   */           
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_Att (ViSession instrID, ViBoolean controlType,
                                                   ViReal64 inputAttValue)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
       
    if (avr3132s_invalidViBooleanRange (controlType))             
        return VI_ERROR_PARAMETER2;   

    if (controlType == VI_ON) { /* attenuation chosen by the analyzer */ 
        Fmt (wrtBuffer,"%s<%s\r\n", s_att[controlType]);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
             return avr3132s_status;
    }
    else { /* attenuation chosen by the user */
        if (avr3132s_invalidViReal64Range (inputAttValue, featuresPtr->inputAttMin,
                                                      featuresPtr->inputAttMax))             
            return VI_ERROR_PARAMETER3; 
        Fmt (wrtBuffer,"%s<%s %fDB\r\n",s_att[controlType],inputAttValue);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;
    }
     
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read Attenuation                                                */
/* Purpose:  returns the input attenuation and says whether this parameter   */
/* is controled  by the user or by the analyzer                              */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_Attenuation (ViSession instrID,ViPReal64 value,
                                                           ViPInt16 controlType)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    /* returns the value of the input attenuation */
    Fmt (Buffer,"%s<%s\r\n",s_readAtt[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f", value)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
     
    /* returns whether the input attenuation is controlled by the analyser or by the user */
    Fmt (Buffer,"%s<%s\r\n",s_readAtt[1]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", controlType)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
      
    return avr3132s_status;
}
    
/*===========================================================================*/
/* Function: Set RBW                                                         */           
/* Purpose:  Set up the Resolution Bandwidth.                                */
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_RBW (ViSession instrID, ViInt16 RBWsetting)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (avr3132s_invalidViInt16Range (RBWsetting,0,8))             
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuffer,"%s<%s\r\n",s_RBW[RBWsetting]);     
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
       
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set VBW                                                         */           
/* Purpose:  Set up the Video Bandwidth.                                     */
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_VBW (ViSession instrID, ViInt16 VBWsetting)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (avr3132s_invalidViInt16Range (VBWsetting,0,12))             
        return VI_ERROR_PARAMETER2;
       
    Fmt (wrtBuffer,"%s<%s\r\n",s_VBW[VBWsetting]);     
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;    

    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set Wide RBW                                                    */           
/* Purpose:  This function switches the wide band RBW.						 */
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_wideRBW (ViSession instrID, ViBoolean wideRBW)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (avr3132s_invalidViBooleanRange (wideRBW))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer,"%s<%s",s_wideRBW[wideRBW]);     
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;    

    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set All Auto Couple                                             */           
/* Purpose:  Sets all couple functions to automatic mode                     */
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_All_Auto_Couple (ViSession instrID)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    Fmt (wrtBuffer,"%s<%s\r\n",s_All_Coupled[0]);     
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status; 
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read RBW and VBW                                                */
/* Purpose:  reads the resolution or video bandwith and says whether this    */
/*           parameter is controlled by the analyzer or by the user          */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_RBWandVBW (ViSession instrID, ViBoolean parameter,
                                                          ViPReal64 value,
                                                          ViPInt16 controlType)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViBooleanRange (parameter))             
        return VI_ERROR_PARAMETER2;
        
    /* returns the value of the resolution or video bandwidth */
    Fmt (Buffer,"%s<%s\r\n",s_readRBWandVBW[parameter]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f", value)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    /* returns whether the video or resolution bandwidth is controlled by the analyser or by the user */
    Fmt (Buffer,"%s<%s\r\n",s_readRBWandVBW[parameter+2]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", controlType)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    } 
     
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set Auto Manual                                                 */           
/* Purpose:  This function selects item, and sets its condition.             */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_AutoManual (ViSession instrID,
                                       ViInt16 contents, ViBoolean mode,
                                       ViReal64 manualValue)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViInt16Range (contents, 0, 4))             
        return VI_ERROR_PARAMETER2; 
    if (avr3132s_invalidViBooleanRange (mode))             
        return VI_ERROR_PARAMETER3;
    
    if (mode==VI_ON)
    	Fmt (Buffer, "%s<%s", s_setAuto[contents]);
    else
    	Fmt (Buffer, "%s<%s %f[e]%s", s_setManual[contents], manualValue, s_unitManual[contents]);
    	
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
        
    return avr3132s_status;    
}

/*===========================================================================*/
/* Function: Set sweep time                                                  */           
/* Purpose:  Sets the sweep time (sec)                                       */
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_SweepTime (ViSession instrID, ViBoolean controlType,
                                                         ViReal64 sweepTime)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
       
    if (avr3132s_invalidViBooleanRange (controlType))             
        return VI_ERROR_PARAMETER2;
        
    if (controlType == VI_ON) { /*sweep time is chosen by the analyzer*/  
        Fmt (wrtBuffer,"%s<%s\r\n",s_sweepTime[controlType]);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;  
    }
    else { /*sweep time is chosen by the user*/ 
        if (avr3132s_invalidViReal64Range (sweepTime,featuresPtr->sweepTimeMini,
                                                 featuresPtr->sweepTimeMaxi))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s%fSC\r\n",s_sweepTime[controlType],sweepTime);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;  
    }
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read  Sweep Time                                                */
/* Purpose:  reads the sweep time value and says whether we are in auto or   */
/*           manual control                                                  */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_SweepTime (ViSession instrID, ViPReal64 timeValue,
                                                          ViPInt16 controlType)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    /* returns the value of the sweep time */
    Fmt (Buffer,"%s<%s\r\n",s_readSweepTime[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f",timeValue)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    /* returns whether the sweep time is controlled by the analyser or by the user */
    Fmt (Buffer,"%s<%s\r\n",s_readSweepTime[1]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", controlType)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    } 
     
    return avr3132s_status;
}        

/*===========================================================================*/
/* Function: Set sweep mode                                                  */           
/* Purpose:  Sets the sweep mode: continuous/single, window sweep on/off,	 */
/*			 gated sweep on/off												 */
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_SweepMode (ViSession instrID, 
										  ViBoolean sweepMode, ViBoolean windowSweep)
{
    ViStatus avr3132s_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (avr3132s_invalidViBooleanRange (sweepMode))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViBooleanRange (windowSweep))             
        return VI_ERROR_PARAMETER3;
    
    Fmt (wrtBuffer,"%s<%s; %s", s_windowSweep[windowSweep], s_sweepMode[sweepMode]);     
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
     
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Execute X dB down                                               */
/* Purpose:  This function sets storage type of the frequency and level of   */
/*			 displayed delta marker.			                             */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Execute_XdBDown (ViSession instrID, ViInt16 dBDown)
{
	ViStatus avr3132s_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (avr3132s_invalidViInt16Range (dBDown, 0, 2))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer, "%s<%s", s_exeXdBDown[dBDown]);
    
    if ((avr3132s_status = viWrite(instrID, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avr3132s_status;
     
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Execute 3rd Order Measurement                                   */
/* Purpose:  This function displays the delta marker on the peak of the		 */
/*			 fundamental wave and the normal marker on the peak of the third */
/*			 order intermodulation distortion.                               */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Execute_3rdOrder (ViSession instrID)
{
	ViStatus avr3132s_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    Fmt (wrtBuffer, "%s<PKTHIRD");
    
    if ((avr3132s_status = viWrite(instrID, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avr3132s_status;
     
    return avr3132s_status;    
}

/*===========================================================================*/
/* Function: Execute AM Modulation Measurement                               */
/* Purpose:  This function switches AM modulation measurement ON or OFF.     */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Execute_AMModulation (ViSession instrID, ViBoolean measAM)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (avr3132s_invalidViBooleanRange (measAM))             
        return VI_ERROR_PARAMETER2;
    Fmt (wrtBuffer, "%s<%s", s_exeAMMeas[measAM]);
    
    if ((avr3132s_status = viWrite(instrID, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avr3132s_status;
     
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read AM Modulation Measurement                                  */
/* Purpose:  This function returns the results of the AM Modulation			 */
/*			 measurement.                               					 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_AMModulation (ViSession instrID, ViReal64 *res)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if ((avr3132s_status = viWrite(instrID, "AMMOD?", 6, &retCount)) < 0)
        return avr3132s_status;
        
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
        
    if (Scan (Buffer, "%s>%f", res) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	
   	return avr3132s_status;	   		
}

/*===========================================================================*/
/* Function: Take Sweep                                                      */
/* Purpose:  Sets sweep mode & starts sweeping                               */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Take_Sweep (ViSession instrID, ViInt16 startSweep)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
  
    if (avr3132s_invalidViInt16Range (startSweep,0,3))             
        return VI_ERROR_PARAMETER2;
     
    Fmt (wrtBuffer,"%s<%s\r\n",s_sweepMode[startSweep]);   
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;   
       
    return avr3132s_status;
}  

/*===========================================================================*/
/* Function: Read  Sweep Mode                                                */
/* Purpose:  returns the sweep Mode: Normal&Full, Single&Full, Normal&Window */
/*           or Single&Window                                                */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_SweepMode (ViSession instrID, ViPInt16 status,
                                                          ViChar _VI_FAR sweepMode[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readSweepMode[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
     
    if (*status==0)
        Fmt(sweepMode,"%s<%s",s_readSweepMode[1]);
    else if (*status==20)
        Fmt(sweepMode,"%s<%s",s_readSweepMode[2]);   
    else if (*status==1)
        Fmt(sweepMode,"%s<%s",s_readSweepMode[3]); 
    else if (*status==21)
        Fmt(sweepMode,"%s<%s",s_readSweepMode[4]);
     
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set trigger                                                     */           
/* Purpose:  Defines the trigger source , the polarity of the video and      */
/*           external triggers,the levels at which the video and	         */
/*           external trigger occurs, and the trigger delay.                 */
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_Trigger(ViSession instrID, ViInt16 source,
                                   ViBoolean polarity, ViInt16 videoLevel,
                                   ViReal64 externalLevel,
                                   ViBoolean trgDly, ViReal64 dlyTime)
{
    ViStatus avr3132s_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;  
    ViUInt32 retCount = 0;
    ViUInt32 total = 0;
    ViInt16 instrument;

    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    
    instrument = featuresPtr->instrumentType;
    if (instrument == 3131) instrument = 0; else instrument = 1;

    if (avr3132s_invalidViInt16Range (source,0,3))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViBooleanRange (polarity))              
        return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViBooleanRange (trgDly))
    	return VI_ERROR_PARAMETER6;

    polarity += 3;
    if (source<=1)  /* free run and line */
        Fmt (wrtBuffer,"%s<%s",s_trigger[source + 6*instrument]);     
    else if (source==2) { /* video trigger */
        if (avr3132s_invalidViInt16Range (videoLevel,0,100))             
            return VI_ERROR_PARAMETER4;
        Fmt (wrtBuffer,"%s<%s %d[b4]ENT,%s",s_trigger[source],videoLevel,s_trigger[polarity]);     
    }
    else if (source==3) {  /* external trigger */
        if (avr3132s_invalidViInt16Range (externalLevel,0,5))             
            return VI_ERROR_PARAMETER5;
        Fmt (wrtBuffer,"%s<%s %fENT,%s",s_trigger[source+=2],externalLevel,s_trigger[polarity]);     
    }
    total += NumFmtdBytes();
    
    if (trgDly==VI_ON)
    {
    	Fmt (wrtBuffer, "%s[a]<; TRGDT %f[e]SC", dlyTime);
    	total += NumFmtdBytes();
    }
    
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,total,&retCount)) < 0)
        return avr3132s_status;

    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set trigger TV                                                  */           
/* Purpose:  Defines the trigger TV source,the polarity of this trigger      */
/*           and the line number at which the trigger occurs                 */
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_TriggerTV(ViSession instrID, ViInt16 triggerSource,
                                                        ViBoolean triggerPolarity,
                                                        ViInt16 lineNumber)
{
    ViStatus avr3132s_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (avr3132s_invalidViInt16Range (triggerSource,0,2))             
        return VI_ERROR_PARAMETER2;
        
    if (avr3132s_invalidViBooleanRange (triggerPolarity))             
        return VI_ERROR_PARAMETER3;
        
    if (triggerSource != 0) { /* TV-H */
        if (avr3132s_invalidViInt16Range (lineNumber,1,525))             
            return VI_ERROR_PARAMETER4;
        Fmt (wrtBuffer,"%s<%s,%s,%s%d[b2]ENT,%s\r\n",s_triggerTV[1],s_triggerTV[triggerSource + 1],
                                                     s_triggerTV[4],lineNumber,s_triggerTV[5+triggerPolarity]);     
    }
    else { /* TV-V */
        Fmt (wrtBuffer,"%s<%s,%s\r\n",s_triggerTV[triggerSource],
                                      s_triggerTV[5+triggerPolarity]);
    }
                                                  
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status; 

    return avr3132s_status;
} 
 
/*===========================================================================*/
/* Function: Read Trigger                                                    */
/* Purpose:  returns the information about the trigger                       */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_Trigger (ViSession instrID, ViPInt16 triggerStatus,
                                                        ViChar _VI_FAR triggerName[],
                                                        ViPInt16 triggerSlope,
                                                        ViPReal64 triggerFeature)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 index=0, indextab[6]= {0,0,3,0,4,5} ;
    avr3132s_ptrFeatures featuresPtr;  
    ViInt16 instrument;

    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    
    instrument = featuresPtr->instrumentType;
    if (instrument == 3131) instrument = 0; else instrument = 1;
   
    /*searchs the trigger used*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readTrigger[instrument]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
        
    if (Scan(Buffer,"%s>%d[b2]", triggerStatus)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    Fmt(triggerName,"%s<%s",s_triggerName[*triggerStatus]);
    
    /* returns the features of the  TV-H, Video or External trigger */
    
    if ((*triggerStatus==2)||(*triggerStatus==4)||(*triggerStatus==5)) {
     
       index = indextab[*triggerStatus];
       
       Fmt (Buffer,"%s<%s\r\n",s_readTrigger[index]);
       if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
       	   return avr3132s_status;
    
       if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
       	   return avr3132s_status;
        
    	if (Scan(Buffer,"%s>%f", triggerFeature)!=1) { 
        	avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        	return avr3132s_status;
     	} 
       
    
    }   
    
    /* returns the trigger slope */ 
    
    Fmt (Buffer,"%s<%s\r\n",s_readTrigger[2]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
        
    if (Scan(Buffer,"%s>%d[b2]", triggerSlope)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }   
     
    return avr3132s_status;
} 
 
/*===========================================================================*/
/* Function: Set Trace Mode                                                  */           
/* Purpose:  Defines Trace A or B mode (Write, View, Max Hold ...)           */
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_TraceMode (ViSession instrID, ViBoolean trace,
                                                         ViInt16 traceMode,
                                                         ViInt16 averageCount,
                                                         ViBoolean averageMode)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    ViInt16 instrument;

    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    
    instrument = featuresPtr->instrumentType;
    if (instrument == 3131) instrument = 0; else instrument = 1;
    
    if (avr3132s_invalidViBooleanRange (trace))             
        return VI_ERROR_PARAMETER2; 
    if (avr3132s_invalidViInt16Range (traceMode, 0, 7))             
        return VI_ERROR_PARAMETER3;
       
    if (traceMode != 7) { /* set trace mode except trace average */ 
        Fmt (wrtBuffer,"%s<%s\r\n",s_traceMode[traceMode + trace*8 + 16*instrument]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
    else { /* set trace average: some parameters are needed */
        if (avr3132s_invalidViInt16Range (averageCount,featuresPtr->averageCountMini,
                                                   featuresPtr->averageCountMaxi))             
            return VI_ERROR_PARAMETER4;
        if (avr3132s_invalidViBooleanRange (averageMode))             
            return VI_ERROR_PARAMETER5;
        Fmt (wrtBuffer,"%s<%s%d[b2]ENT,%s,%s\r\n",s_averageMode[2 + trace*3],averageCount,
                                       s_averageMode[averageMode + trace*3],
                                       s_traceMode[traceMode + trace*8 + 16*instrument]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read Trace Status                                               */
/* Purpose:  returns the status of the trace A or B                          */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_TraceStatus (ViSession instrID, ViBoolean trace,
                                                            ViPInt16 status,
                                                            ViChar _VI_FAR message[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    
    if (avr3132s_invalidViBooleanRange (trace))             
        return VI_ERROR_PARAMETER2; 
 
    Fmt (Buffer,"%s<%s\r\n",s_readTraceStatus[trace]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    Fmt(message,"%s<%s",s_readTraceMessage[*status+trace*6]); 
    return avr3132s_status; 
}

/*===========================================================================*/
/* Function: Set Trace Detector Mode                                         */           
/* Purpose:  Specifies detector mode for the trace A or the trace B          */
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_TraceDetector (ViSession instrID, ViBoolean trace, 
                                                             ViInt16 detectorMode) 
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViBooleanRange (trace))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViInt16Range (detectorMode,0,3))             
        return VI_ERROR_PARAMETER3;

    Fmt (wrtBuffer,"%s<%s\r\n",s_detectorMode[detectorMode + trace*4]);   
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    return avr3132s_status;                                          
}

/*===========================================================================*/
/* Function: Read Detector Mode                                              */
/* Purpose:  returns the detector mode                                       */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_DetectorMode (ViSession instrID, ViBoolean trace,
                                                             ViPInt16 status,
                                                             ViChar _VI_FAR detectorMode[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
        
    if (avr3132s_invalidViBooleanRange (trace))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (Buffer,"%s<%s\r\n",s_readDetectorMode[trace]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
     
    Fmt(detectorMode,"%s<%s",s_readDetectorMode[*status+2+4*trace]); 
    return avr3132s_status; 
}

/*===========================================================================*/
/* Function: Start/Stop Trace Average                                        */
/* Purpose:  Starts stops trace average                                      */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Control_TraceAverage (ViSession instrID, ViBoolean trace,
                                                                ViInt16 traceAverage)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViBooleanRange (trace))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViInt16Range (traceAverage,0,3))             
        return VI_ERROR_PARAMETER3;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_traceAverage[traceAverage + trace*4]);   
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;    
        
    return avr3132s_status;  
}

/*===========================================================================*/
/* Function: Read Trace Data                                                 */
/* Purpose:  returns trace A or B in an array                                */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_TraceData (ViSession instrID, ViBoolean trace,
                                                          ViBoolean format,
                                                          ViInt16 _VI_FAR traceArray[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    avr3132s_ptrFeatures featuresPtr;
    ViInt16 i, instrument, nbpoints;
    ViUInt32 retCount;
    ViByte dataBuffer[2005], wrtBuffer[2*BUFFER_SIZE];
    
    if (avr3132s_invalidViBooleanRange (trace))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViBooleanRange (format))             
        return VI_ERROR_PARAMETER3;
        
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
	
	instrument = featuresPtr->instrumentType;
    if (instrument == 3131) instrument = 0; else instrument = 1;
    
    for (i=0;i<2005;i++) {
        dataBuffer[i]=0;
    }
    
    if (instrument == 1) {
    	avr3132s_Read_TracePoints (instrID, &nbpoints); 
    }
    else {
    	nbpoints = 501;
    }
    
    if (format == VI_ON) { /*Binary Transfer*/
        Fmt (wrtBuffer,"%s<%s\r\n",s_queryBinary[trace]);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;
        if ((avr3132s_status = viRead (instrID,dataBuffer,(2*nbpoints)+2,&retCount)) < 0)
            return avr3132s_status;
        for (i=0;i<nbpoints;i++) {
            traceArray[i]=256*dataBuffer[2*i]+dataBuffer[2*i+1] ;
        }
    }
    else { /*ASCII Transfer*/ 
        Fmt (wrtBuffer,"%s<%s\r\n",s_queryASCII[trace]);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;
        for (i=0;i<nbpoints;i++) {
            if ((avr3132s_status = viRead (instrID, dataBuffer, 7, &retCount)) < 0)
                return avr3132s_status;
            if (Scan(dataBuffer,"%s>%d[b2]", &traceArray[i])!=1) { 
                avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                return avr3132s_status;
            }   
        }
    }   
     
  return avr3132s_status; 
}

/*===========================================================================*/
/* Function: Write Trace Data                                                */
/* Purpose:  write array data to the trace A or B                            */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Write_TraceData (ViSession instrID,
                                        ViBoolean trace, ViBoolean format,
                                        ViInt16 traceArray[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[2100];
    avr3132s_ptrFeatures featuresPtr;
    ViInt16 i, instrument, nbpoints;
    ViUInt32 retCount=0;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
	
	instrument = featuresPtr->instrumentType;
    if (instrument == 3131) instrument = 0; else instrument = 1;
    
    if (instrument == 1)
    	avr3132s_Read_TracePoints (instrID, &nbpoints); 
    else
    	nbpoints = 501;
    	
    if (format == VI_ON) /* Binary Transfer */
    {
    	Fmt (Buffer, "%s<%s", s_traceBinary[trace]);
    	
    	if ((avr3132s_status = viWrite (instrID, Buffer, NumFmtdBytes (), &retCount)) < 0)
   	    	return avr3132s_status;
   	    	
   	    if (nbpoints == 1001)
   			Fmt (Buffer, "%1001d[zb2o10]<%1001d[b2]", traceArray);
   		else
   			Fmt (Buffer, "%501d[zb2o10]<%501d[b2]", traceArray);
   	
    	if ((avr3132s_status = viWrite (instrID, Buffer, NumFmtdBytes (), &retCount)) < 0)
   	    	return avr3132s_status;	
    }
    
    else /* ASCII Transfer */
    {
    	Fmt (Buffer, "%s<%s", s_traceASCII[trace]);
    	
    	if ((avr3132s_status = viWrite (instrID, Buffer, NumFmtdBytes (), &retCount)) < 0)
   	    	return avr3132s_status;
   	    	
   	    for (i=0; i<nbpoints; i++)
   	    {
   	    	Fmt (Buffer, "%s<%d[b2]", *traceArray++);
   	    	
   	    	if ((avr3132s_status = viWrite (instrID, Buffer, NumFmtdBytes (), &retCount)) < 0)
   	    		return avr3132s_status;
   	    }
    }
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set Trace Points                                                */
/* Purpose:  chooses the number of points in a trace                         */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_TracePoints (ViSession instrID, ViBoolean tracePoints)
  
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViBooleanRange (tracePoints))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (Buffer,"%s<%s\r\n",s_tracePoints[tracePoints]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
        
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read Trace Points                                               */
/* Purpose:  returns the number of points in a trace                         */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_TracePoints (ViSession instrID, ViPInt16 tracePoints)
  
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 status;
    
    Fmt (Buffer,"%s<%s\r\n",s_tracePoints[2]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
        
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
       
    if (Scan(Buffer,"%s>%d[b2]", &status)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    if (status==0) *tracePoints = 501;
    else *tracePoints = 1001;
        
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set Normal Marker                                               */
/* Purpose:  sets the normal marker position                                 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_NormalMarker (ViSession instrID,ViReal64 position)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    avr3132s_ptrFeatures featuresPtr;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
        
    Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrNormal[0],position,s_positionUnit[featuresPtr->span]);   
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;   
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set Delta Marker                                                */
/* Purpose:  Sets the delta marker position                                  */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_DeltaMarker (ViSession instrID, ViReal64 position)
{
    ViStatus avr3132s_status = VI_SUCCESS; 
    ViByte wrtBuffer[BUFFER_SIZE];      
    ViUInt32 retCount=0;
    avr3132s_ptrFeatures featuresPtr;
    ViInt16 status, markerStepControl, fixedMarkerStatus, coupleMarkerStatus;
    ViReal64 MKRDpos, MKRDlevel, MKRNpos, MKRNlevel, markerStepValue;
    ViChar message[50];
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
       return avr3132s_status;
       
    // returns the delta marker fixed status 
    avr3132s_Read_MarkerOptions (instrID, &markerStepControl, &markerStepValue, &fixedMarkerStatus,
                                      message, &coupleMarkerStatus, message);
    
    // returns the number of activated marker
    avr3132s_Read_MarkerStatus (instrID, &status, message);
    
    if ( (status == 2)&&(fixedMarkerStatus == 0) ) {  /* normal and delta markers are already active */
        
        avr3132s_Read_DeltaMarker (instrID, &MKRDpos, &MKRDlevel, &MKRNpos, &MKRNlevel);
        
        Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrNormal[0],MKRDpos,s_positionUnit[featuresPtr->span]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
            
        Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrDelta[0],position,s_positionUnit[featuresPtr->span]);   
    	if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        	return avr3132s_status;    
        	
    }
    else if ( (status == 2)&&(fixedMarkerStatus == 1) ) {
    
    	avr3132s_Read_DeltaMarker (instrID, &MKRDpos, &MKRDlevel, &MKRNpos, &MKRNlevel); 
    
    	Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrNormal[0],MKRDpos,s_positionUnit[featuresPtr->span]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
            
    	Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrOptions[3],position,s_positionUnit[featuresPtr->span]);   
    	if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        	return avr3132s_status;
    }
    else {
    	Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrDelta[0],position,s_positionUnit[featuresPtr->span]);   
    	if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        	return avr3132s_status;
    }
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read Marker Status                                              */
/* Purpose:  reads the marker status number --> text                         */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_MarkerStatus (ViSession instrID, ViPInt16 status,
                                                             ViChar _VI_FAR message[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_mkrStatusRead[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
       
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]",status)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }   
    
    Fmt(message,"%s<%s", s_mkrStatus[*status]);  
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: read normal marker                                              */
/* Purpose:  reads the normal marker position and level (without unit)       */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_NormalMarker (ViSession instrID,ViPReal64 MKRNposition,
                                                            ViPReal64 MKRNlevel)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 markerstatus;
    ViChar  markerstatusmessage[40];
    
    /*reads the absolute position of the normal marker */
    Fmt (Buffer,"%s<%s\r\n",s_normalMkrRead[2]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
       
    if (Scan(Buffer,"%s>%f", MKRNposition)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
     
    /*reads the absolute level of the marker normal*/ 
    Fmt (Buffer,"%s<%s\r\n",s_normalMkrRead[3]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f", MKRNlevel)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: read delta marker                                               */
/* Purpose:  reads the position and level (without unit) of the delta marker */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_DeltaMarker (ViSession instrID,ViPReal64 MKRDposition,
                                                           ViPReal64 MKRDlevel,
                                                           ViPReal64 MKRNposition,
                                                           ViPReal64 MKRNlevel)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 offset;
    avr3132s_ptrFeatures featuresPtr;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    
    /* returns the absolute position of the delta marker */
    Fmt (Buffer,"%s<%s\r\n",s_deltaMkrRead[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
        
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f",MKRDposition)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
     
    /* returns the absolute level of the delta marker */
    Fmt (Buffer,"%s<%s\r\n",s_deltaMkrRead[1]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
       
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f",MKRDlevel)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }   

    /* returns the relative position of the normal marker */
    Fmt (Buffer,"%s<%s\r\n",s_normalMkrRead[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
        
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f",MKRNposition)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
     
    /* returns the relative level of the normal marker */
    Fmt (Buffer,"%s<%s\r\n",s_normalMkrRead[1]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
       
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f",MKRNlevel)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }   

    return avr3132s_status;
}

/*===========================================================================*/
/* Function: X dB Down				                                         */
/* Purpose:  This function displays the frequency or level difference 		 */
/*			 between the reference marker and a marker displayed at X dB 	 */
/*			 lower (or higher) position, and moves marker to desired 		 */
/*			 position.                                      				 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_XdBDown (ViSession instrID,
                                    ViReal64 downWidth, ViInt16 mkrRead,
                                    ViBoolean contDown)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViReal64Range (downWidth, -100.0, 100.0))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViInt16Range (mkrRead, 0, 2))             
        return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViBooleanRange (contDown))             
        return VI_ERROR_PARAMETER4;
        
    Fmt (Buffer, "%s<MKBW %fDB; %s; %s", downWidth, s_markerRead[mkrRead], s_contDown[contDown]);
    
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
        
    return avr3132s_status;    
}   


/*===========================================================================*/
/* Function: Activate Multimarkers                                           */
/* Purpose:  activates the multimarkers                                      */
/*===========================================================================*/                                  
ViStatus _VI_FUNC avr3132s_Activate_Multimarkers (ViSession instrID)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_multiMkrStatus[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    return avr3132s_status;                              
} 

/*===========================================================================*/
/* Function: Set Multimarkers                                                */
/* Purpose:  sets the Multi_Markers (frequency or time)                      */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_Multimarkers (ViSession instrID, ViInt16 multiMkrStatus[],
                                                            ViReal64 multiMkrPositions[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 i=0;
    avr3132s_ptrFeatures featuresPtr;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    
    for (i=0; i<10; i++) {
    	featuresPtr->MKRMstatus[i] = multiMkrStatus[i];
    
        if (multiMkrStatus[i]!= 0) {
            Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_multiMkrON[i],multiMkrPositions[i],s_positionUnit[featuresPtr->span]);   
            if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
                return avr3132s_status;
        }
        else {
            Fmt (wrtBuffer,"%s<%s\r\n",s_multiMkrOFF[i]);   
            if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
                return avr3132s_status;
        }
    }
    
    return avr3132s_status; 
    
}

/*===========================================================================*/
/* Function: Set ActiveMultimarker                                           */
/* Purpose:  chooses the active Multi-Marker                                 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_ActiveMultimarker (ViSession instrID, ViInt16 index)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 activeMultimarker, status = 0;
    ViChar message[100];
    ViReal64 MKRMpositions[10];
    ViReal64 MKRDposition, MKRNposition, deltaPosition;
    ViReal64 MKRMlevels[10];
    ViReal64 MKRDlevel, MKRNlevel;
    
    avr3132s_ptrFeatures featuresPtr;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
        
    if (avr3132s_invalidViInt16Range (index,1,10))             
        return VI_ERROR_PARAMETER2;
        
    avr3132s_Read_MarkerStatus (instrID, &status, message);       
    
    avr3132s_Read_MultimarkerActive (instrID, &activeMultimarker);
    
    avr3132s_Read_NormalMarker (instrID, &MKRNposition, &MKRNlevel);
    
    avr3132s_Read_Multimarkers (instrID, MKRMpositions,&MKRDposition,MKRMlevels,&MKRDlevel);
                                       
    deltaPosition = (MKRNposition - (status - 1)*MKRDposition) - MKRMpositions[activeMultimarker - 1];                                   
                                       
    if (featuresPtr->MKRMstatus[index-1]!= 0) {                                   
        Fmt (Buffer,"%s<%s %f%s\r\n",s_multiMkrON[index-1],MKRMpositions[index-1] + (status - 1)*deltaPosition,s_positionUnit[featuresPtr->span]);   
        if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status; 
    }
    
    else {
        return VI_ERROR_MARKERINACTIVE;
    }   
    
    return avr3132s_status; 
}

/*===========================================================================*/
/* Function: Delete All Markers                                              */
/* Purpose:  deletes the markers which are active                            */
/*===========================================================================*/                                  
ViStatus _VI_FUNC avr3132s_Delete_AllMarkers (ViSession instrID)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_deleteMarker[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    return avr3132s_status;                              
} 

/*===========================================================================*/
/* Function: Delete Multi Markers                                            */
/* Purpose:  deletes the multi markers which are active                      */
/*===========================================================================*/                                  
ViStatus _VI_FUNC avr3132s_Delete_Multimarkers (ViSession instrID)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_deleteMarker[1]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    return avr3132s_status;                              
} 

/*===========================================================================*/
/* Function: Read Multi Marker Status                                        */
/* Purpose:  reads the multi marker status & gives text message              */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_MultimarkersStatus (ViSession instrID,ViPInt16 status,
                                                                  ViChar _VI_FAR message[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readmultiMkrStatus[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
       
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
       
    if (Scan(Buffer,"%s>%d[b2]",status)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }   
    
    Fmt(message,"%s<%s", s_multiMkrMessage[*status]);  
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read Multi Markers                                              */
/* Purpose:  reads the positions and levels of the 10 multimarkers           */
/*           and the delta marker                                            */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_Multimarkers (ViSession instrID,ViReal64 _VI_FAR MKRMpositions[10],
                                                            ViPReal64 MKRDposition,
                                                            ViReal64 _VI_FAR MKRMlevels[10],
                                                            ViPReal64 MKRDlevel)
                                          
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViInt16 i, result;
    ViUInt32 retCount=0;
    
    /* returns the multimarkers positions */
    
    Fmt (Buffer,"%s<%s\r\n",s_multiMkrSearch[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;

	if ((avr3132s_status = viRead (instrID, Buffer, 7, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", &result)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
	}
	
    for (i=0;i<10;i++) {
        if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
            return avr3132s_status;
        if (Scan(Buffer,"%s>%f", &MKRMpositions[i])!=1) { 
            avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            return avr3132s_status;
		}
	}
	
	if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f", MKRDposition)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;	
    }    

    /* returns the multimarkers levels*/
    
    Fmt (Buffer,"%s<%s\r\n",s_multiMkrSearch[1]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 7, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", &result)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }    

    for (i=0;i<10;i++) {
        if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
            return avr3132s_status;
        if (Scan(Buffer,"%s>%f", &MKRMlevels[i])!=1) { 
            avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            return avr3132s_status;
		}
	}
	
	if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f", MKRDlevel)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }    
       
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read MultimarkerActive                                          */
/* Purpose:  searches the active Multimarker                                 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_MultimarkerActive (ViSession instrID, ViPInt16 index)
                                      
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt32 bytesRead;
    ViInt16 i = 0, markerStatus, fixedMarkerStatus, markerStepControl, coupleMarkerStatus;
    ViReal64 MKRNposition, delta, deltaMin = 10e9;
    ViReal64 MKRMpositions[10];
    ViReal64 MKRDposition;
    ViReal64 MKRMlevels[10];
    ViReal64 MKRDlevel, MKRstepsize;
    ViChar message[40];
    ViReal64 MKRNpos, MKRDpos, level, markerStepValue;
    
    avr3132s_Read_MarkerStatus (instrID, &markerStatus, message);
    avr3132s_Read_MarkerOptions (instrID, &markerStepControl, &markerStepValue, &fixedMarkerStatus,
                             message, &coupleMarkerStatus, message);
    avr3132s_Read_Multimarkers (instrID, MKRMpositions, &MKRDposition, MKRMlevels, &MKRDlevel);  
    
    if ( (markerStatus == 2)&&(fixedMarkerStatus == 1) ) {  /* fixed delta markers */
    
    	// MKR step size = 0
    	avr3132s_Set_FreqStepSize (instrID, 0, 0);
    	
    	// MKR -> MKR step size
    	avr3132s_Write_InstrData (instrID, s_markerStepSize[0]);
    	
    	// returns the new MKR step size
    	avr3132s_Write_InstrData (instrID, s_markerStepSize[1]);
    	avr3132s_Read_InstrData (instrID, 50, Buffer, &bytesRead);
    	Scan(Buffer,"%s>%f",&MKRstepsize);
                                                           
    	// Compare MKR step size and MKRMpositions[i] to determine the active marker 
    	for (i = 0; i < 10; i++) {
    		delta = fabs ((MKRDposition + MKRMpositions[i]) - MKRstepsize) ;
    		if ( delta < deltaMin ) {
    			deltaMin = delta;
    			*index = i + 1;
    		}
    	}
    	
    	// MKR step size = AUTO
    	avr3132s_Set_FreqStepSize (instrID, 1, 0);
    
    }    
    
    else {
    
    	Fmt (Buffer,"%s<%s\r\n",s_normalMkrRead[0]);   
    	if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        	return avr3132s_status;
    
    	if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        	return avr3132s_status;
       
	    if (Scan(Buffer,"%s>%f", &MKRNposition)!=1) { 
	        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	        return avr3132s_status;
	    }
     
	    while ( (i<10)&&((MKRMpositions[i]!=MKRNposition)||(MKRMlevels[i]==0)) ) {
	        i++;
	    } 
    
   		*index = i+1;
   		
   	}	
    
    return avr3132s_status; 
}

/*===========================================================================*/
/* Function : Set Marker Trace                                               */
/* Purpose : chooses the trace on which the markers will be active           */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_MarkerTrace (ViSession instrID, ViBoolean markerTrace)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViBooleanRange (markerTrace))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (Buffer,"%s<%s\r\n",s_mkrTrace[markerTrace]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
       
    return avr3132s_status;
} 

/*===========================================================================*/
/* Function: Read Marker Trace Status                                        */
/* Purpose:  returns the trace on which the markers are placed               */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_MarkerTraceStatus (ViSession instrID,ViPInt16 status,
                                                                 ViChar _VI_FAR message[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    
    Fmt (Buffer,"%s<%s\r\n",s_readMkrTrace[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
     
    Fmt(message,"%s<%s",s_readMkrTrace[*status+1]); 
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set marker options                                              */
/* Purpose:  marker step size, fixed delta marker, couple to marker          */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_MarkerOptions (ViSession instrID,ViBoolean stepstatus,
                                                            ViReal64 stepvalue,     
                                                            ViBoolean fixed,
                                                            ViBoolean couple)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];      
    ViUInt32 retCount=0;
    avr3132s_ptrFeatures featuresPtr;
    ViInt16 status;
    ViChar message[50];
    ViReal64 MKRNposition, MKRNlevel;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;

    if (avr3132s_invalidViBooleanRange (stepstatus))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViReal64Range (stepvalue,featuresPtr->centerLow,featuresPtr->centerHigh))             
        return VI_ERROR_PARAMETER3;   
    if (avr3132s_invalidViBooleanRange (fixed))             
        return VI_ERROR_PARAMETER4;
    if (avr3132s_invalidViBooleanRange (couple))             
        return VI_ERROR_PARAMETER5;
        
    avr3132s_Read_MarkerStatus (instrID, &status, message);   
    
    if ((fixed == VI_OFF)&&(status == 0)) {
        Fmt (wrtBuffer,"%s<%s\r\n",s_mkrOptions[fixed+2]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
        avr3132s_Delete_AllMarkers (instrID);          
    }
    else if ((fixed == VI_OFF)&&(status == 1)) {
        avr3132s_Read_NormalMarker (instrID, &MKRNposition, &MKRNlevel);
        Fmt (wrtBuffer,"%s<%s\r\n",s_mkrOptions[fixed + 2]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
        avr3132s_Delete_AllMarkers (instrID);
        avr3132s_Set_NormalMarker (instrID, MKRNposition);
    }
    else if ((fixed == VI_OFF)&&(status==2)) {
        Fmt (wrtBuffer,"%s<%s\r\n",s_mkrOptions[fixed + 2]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
    else if (fixed = VI_ON ) {
        Fmt (wrtBuffer,"%s<%s\r\n",s_mkrOptions[fixed + 2]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
    
    if (stepstatus == VI_ON) {
        Fmt (wrtBuffer,"%s<%s,%s\r\n",s_mkrOptions[stepstatus],s_mkrOptions[couple + 4]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
    else {
        Fmt (wrtBuffer,"%s<%s%f,%s\r\n",s_mkrOptions[stepstatus], stepvalue, s_mkrOptions[couple + 4]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read Marker Options                                             */
/* Purpose:  returns the information about the marker options                */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_MarkerOptions (ViSession instrID, ViPInt16 markerStepControl,
                                                              ViPReal64 markerStepValue,
                                                              ViPInt16 fixedMarkerStatus,
                                                              ViChar _VI_FAR fixedMarkerMessage[],
                                                              ViPInt16 coupleMarkerStatus,
                                                              ViChar _VI_FAR coupleMarkerMessage[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViUInt32 retCount=0;
    ViByte Buffer[BUFFER_SIZE];
    
    /* about the marker step control type */
    Fmt (Buffer,"%s<%s\r\n",s_readMkrOptions[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", markerStepControl)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    /* about the marker step value */
    Fmt (Buffer,"%s<%s\r\n",s_readMkrOptions[1]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f", markerStepValue)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    /*about the fixed delta marker*/
    Fmt (Buffer,"%s<%s\r\n",s_readMkrOptions[2]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", fixedMarkerStatus)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
     
    Fmt(fixedMarkerMessage,"%s<%s",s_textONOFF[*fixedMarkerStatus]);
    
    /*about couple marker*/
    Fmt (Buffer,"%s<%s\r\n",s_readMkrOptions[3]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2x]",coupleMarkerStatus)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    Fmt(coupleMarkerMessage,"%s<%s",s_textONOFF[*coupleMarkerStatus]);
    return avr3132s_status; 
}    

/*===========================================================================*/
/* Function: Set Peak Search Options                                         */
/* Purpose:  Sets the features of the peak search                            */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_PeakSearchOptions (ViSession instrID,ViBoolean continuousSearch,
                                                                ViBoolean signalTrack,
                                                                ViInt16 peakRange,
                                                                ViReal64 deltaY)
{
    ViStatus avr3132s_status = VI_SUCCESS; 
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    avr3132s_ptrFeatures featuresPtr;
    ViInt16 instrument;

    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;

    if (avr3132s_invalidViBooleanRange (continuousSearch))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViBooleanRange (signalTrack))             
        return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViInt16Range (peakRange,0,2))             
        return VI_ERROR_PARAMETER4;
    if (avr3132s_invalidViReal64Range(deltaY,featuresPtr->deltaYmin,
                                         featuresPtr->deltaYmax))
        return VI_ERROR_PARAMETER5;
    
        
    instrument = featuresPtr->instrumentType;
    if (instrument == 3131) instrument = 0; else instrument = 1;    
        
    Fmt (wrtBuffer,"%s<%s,%s,%s,%s%fENT\r\n",s_continuousSearch[continuousSearch + 2*instrument],
                                             s_signalTrack[signalTrack + 2*instrument],
                                             s_peakRange[peakRange],s_peakRange[3],deltaY);   
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
       
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read Peak Search Options                                        */
/* Purpose:  reads the features of the peak search                           */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_PeakSearchOptions (ViSession instrID, ViPInt16 contPeakStatus,
                                                                  ViPInt16 signalTrackStatus,
                                                                  ViPReal64 deltaY)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    /*reads the continuous peak search status*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readPeakOptions[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
        
    if (Scan(Buffer,"%s>%d[b2]", contPeakStatus)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    /*reads the signal track status*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readPeakOptions[1]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
        
    if (Scan(Buffer,"%s>%d[b2]", signalTrackStatus)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    /*reads the Peak DeltaY value*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readPeakOptions[2]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
        
    if (Scan(Buffer,"%s>%f", deltaY)!=1) {                                  
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
     
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Annotatin                                                       */
/* Purpose:  This function switches annotation display.						 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_Annotation (ViSession instrID, ViBoolean annotation)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViBooleanRange (annotation))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer, "%s<%s", s_annotation[annotation]);
    
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    return avr3132s_status;    
}

/*===========================================================================*/
/* Function: Set Mixer Parameters                                            */
/* Purpose:  This function sets the mixer's parameters.						 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_MixerPara (ViSession instrID,
                                          ViBoolean mixer, ViInt16 bndN,
                                          ViBoolean bndLock, ViBoolean avgLoss,
                                          ViReal64 avgLssVl, ViBoolean lossFreq)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViUInt32 total = 0;
    
    if (avr3132s_invalidViBooleanRange (mixer))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViInt16Range (bndN, 1, 14))             
        return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViBooleanRange (bndLock))             
        return VI_ERROR_PARAMETER4;
    if (avr3132s_invalidViBooleanRange (avgLoss))             
        return VI_ERROR_PARAMETER5;
    if (avr3132s_invalidViReal64Range (avgLssVl, -100.0, 100.0))             
        return VI_ERROR_PARAMETER6;
    if (avr3132s_invalidViBooleanRange (lossFreq))             
        return VI_ERROR_PARAMETER7;
    
    if (mixer==VI_ON)   /* Mixer : Internal */
    {
    	Fmt (wrtBuffer, "%s<MXI");
    	total += NumFmtdBytes();
    }
    
    else                /* Mixer : External */
    {
    	Fmt (wrtBuffer, "%s<MXE; BND %d; %s; %s", bndN, s_bandLock[bndLock],
    		 s_lossFreq[lossFreq]);
    	total += NumFmtdBytes();
    	
    	if (avgLoss==VI_ON)
    		Fmt (wrtBuffer, "%s[a]<; AGL ON %f", avgLssVl);
    	else
    		Fmt (wrtBuffer, "%s[a]<; AGL OFF");
    	total += NumFmtdBytes();	
    }
    
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,total,&retCount))<0)
        return avr3132s_status;
    
    return avr3132s_status;    
}

/*===========================================================================*/
/* Function: External Mixer Loss                                             */
/* Purpose:  This function edits loss vs frequency table.					 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_EXTMixerLoss (ViSession instrID,
                                             ViInt16 tblSize, ViReal64 freq[],
                                             ViReal64 lvlMinus[], ViReal64 lvlPlus[])
{
	ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViUInt32 total = 0;
    ViInt16  n;
    ViReal64 *pt_freq, *pt_lvlMns, *pt_lvlPls;
    
    if (avr3132s_invalidViInt16Range (tblSize, 0, 50))             
        return VI_ERROR_PARAMETER2;
    
    n = tblSize;
    pt_freq   = freq;
    pt_lvlMns = lvlMinus;
    pt_lvlPls = lvlPlus;
    while (--n>=0)
    {
    	if (avr3132s_invalidViReal64Range (*pt_freq++, 10.0e9, 400.0e9))
    		return VI_ERROR_PARAMETER3;
    	if (avr3132s_invalidViReal64Range (*pt_lvlMns++, -100.0, 100.0))
    		return VI_ERROR_PARAMETER4;
    	if (avr3132s_invalidViReal64Range (*pt_lvlPls++, -100.0, 100.0))
    		return VI_ERROR_PARAMETER5;
    }
    
    Fmt (wrtBuffer, "%s<LVFDEL");
    total += NumFmtdBytes();
    
    n = tblSize;
    while (--n>=0)
    {
    	Fmt (wrtBuffer, "%s[a]<; LVFIN %fHZ,%fDB,%fDB",
    		 *freq++, *lvlMinus++, *lvlPlus++);
    	total += NumFmtdBytes();	 
    }
    
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,total,&retCount))<0)
        return avr3132s_status;
    
    return avr3132s_status;    	
}

/*===========================================================================*/
/* Function: Search Peak                                                     */
/* Purpose:  executes the selected peak search                               */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Search_Peak (ViSession instrID, ViInt16 typeSearch,
                                                       ViPReal64 deltaPosition,
                                                       ViPReal64 deltaLevel)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViReal64 deltaMarkerPosition, deltaMarkerLevel;
    
    if (avr3132s_invalidViInt16Range (typeSearch,0,5))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_peakSearch[typeSearch]);   
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
        
    if (typeSearch == 5) { /* Peak Max-Min */
        avr3132s_Read_DeltaMarker (instrID, &deltaMarkerPosition, &deltaMarkerLevel,
                                        deltaPosition, deltaLevel);      
    }
       
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Peak List		                                                 */
/* Purpose:  This function switches peak list display, and lists up 		 */
/*			 frequency and level of all marker positions.                    */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_PeakList (ViSession instrID, 
                                 ViInt16 peakList, ViInt16 *numPkList,
                                 ViReal64 frequency[], ViReal64 level[], 
                                 ViInt16 *levelUnits)
{
    ViStatus avr3132s_status = VI_SUCCESS; 
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 i;
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViInt16Range (peakList, 0, 3))             
        return VI_ERROR_PARAMETER2;
        
    if (peakList==3)  /* Peak List Output */
    {
    	if ((avr3132s_status = viWrite(instrID, "PKLST?", 6, &retCount))<0)
        	return avr3132s_status;
        
        if ((avr3132s_status = viRead(instrID, Buffer, 5, &retCount))<0)
        	return avr3132s_status;
        	
        if (Scan (Buffer, "%s>%d[b2]", numPkList) < 1)
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   		
   	   	for (i=0; i < *numPkList; i++)
   	   	{
   	   		if ((avr3132s_status = viRead (instrID, Buffer, 51, &retCount)) < 0)
   	   			return avr3132s_status;
	   	
	   		if (Scan (Buffer, "%s>%f,%f", frequency++, level++) < 2)
   		   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   	}
   	   	
   	   	if ((avr3132s_status = viWrite(instrID, "AUNITS?", 7, &retCount))<0)
        	return avr3132s_status;
        
        if ((avr3132s_status = viRead(instrID, Buffer, BUFFER_SIZE, &retCount))<0)
        	return avr3132s_status;
        	
        if (Scan (Buffer, "%s>%d[b2]", levelUnits) < 1)
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
    
    else
    {
    	Fmt (Buffer, "%s<%s", s_peakList[peakList]);
    	
    	if ((avr3132s_status = viWrite(instrID, Buffer, NumFmtdBytes(), &retCount))<0)
        	return avr3132s_status;
    }
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: execute Peak to                                                 */
/* Purpose:  This function moves the marker to the maximum peak and to 		 */
/*			 assign the peak frequency or level to the Center Frequency or   */
/*			 the Reference Level.                                            */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Execute_PeakTo (ViSession instrID, ViBoolean peakTo)
{
    ViStatus avr3132s_status = VI_SUCCESS; 
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViBooleanRange (peakTo))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_peakTo[peakTo]);   
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
       
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Execute Normal Marker to                                        */
/* Purpose:  This function is used to set the frequency or level of the 	 */
/*			 normal marker as one of the following parameters 				 */
/*			 (center frequency, reference level, step size for the center    */
/*			 frequency or the step size of the marker).                      */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Execute_NormalMarkerTo (ViSession instrID, ViInt16 normalTo)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (avr3132s_invalidViInt16Range (normalTo,0,3))             
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuffer,"%s<%s\r\n",s_normalMarkerTo[normalTo]);   
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    return avr3132s_status;
}
 
/*===========================================================================*/
/* Function: EXecute Marker Delta to                                         */
/* Purpose:  This function set thes difference of frequency between the 	 */
/*			 delta and normal marker as one of the following parameters		 */
/*			 (center frequency, frequency span, step size for the center 	 */
/*			 frequency or the step size of the marker).                      */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Execute_DeltaMarkerTo (ViSession instrID, ViInt16 deltaTo)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (avr3132s_invalidViInt16Range (deltaTo,0,3))             
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuffer,"%s<%s\r\n",s_deltaMarkerTo[deltaTo]);   
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set Gate Parameters	                                         */
/* Purpose:  This function sets gate parameters.							 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_Gate (ViSession instrID,
                                 ViReal64 gatePos, ViReal64 gateWdth,
                                 ViBoolean gateSlp, ViInt16 gateSrc,
                                 ViReal64 extTrgLvl, ViInt16 numTVH,
                                 ViBoolean extGateIn, ViBoolean gatedSweep)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    ViUInt32 total = 0;
    
    if (avr3132s_invalidViReal64Range (gatePos, 0.0, 1.0))             
    	return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViReal64Range (gateWdth, 0.0, 1.0))             
        return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViBooleanRange (gateSlp))             
        return VI_ERROR_PARAMETER4;
    if (avr3132s_invalidViInt16Range (gateSrc, 0, 2))             
    	return VI_ERROR_PARAMETER5;
    if (avr3132s_invalidViReal64Range (extTrgLvl, 0.0, 5.0))             
    	return VI_ERROR_PARAMETER6;
    if (avr3132s_invalidViInt16Range (numTVH, 1, 525))             
    	return VI_ERROR_PARAMETER7;	
    if (avr3132s_invalidViBooleanRange (extGateIn))             
        return VI_ERROR_PARAMETER8;
    if (avr3132s_invalidViBooleanRange (gatedSweep))             
        return VI_ERROR_INSTR_PARAMETER9;    
        
    Fmt (wrtBuffer, "%s<%s; %s; GTPOS %f[e]SC; GTWID %f[e]SC; %s",
    	 s_gateExtIn[extGateIn], s_gatedSweep[gatedSweep], gatePos, gateWdth,
    	 s_gateSlope[gateSlp]);
    total += NumFmtdBytes();
    
    switch (gateSrc)
    {
    case 0: /* EXT Trigger */
    	Fmt (wrtBuffer, "%s[a]<; GTEX %f", extTrgLvl);
    	break;
    	
    case 1: /* TV-V */	
    	Fmt (wrtBuffer, "%s[a]<; GTSRC TVV");
    	break;
    	
    case 2: /* TV-H */
    	Fmt (wrtBuffer, "%s[a]<; GTTVH %d", numTVH);
    	break;
    }
    total += NumFmtdBytes();
    
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,total,&retCount))<0)
        return avr3132s_status;
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set Measurement Window                                          */
/* Purpose:  Sets the position and the width of the measurement window and   */
/*           shows or hides it                                               */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_MeasurementWindow (ViSession instrID,ViBoolean windowStatus,
                                                                ViReal64 Xcenterposition,
                                                                ViReal64 Xwidth)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    avr3132s_ptrFeatures featuresPtr;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    
    if (avr3132s_invalidViBooleanRange (windowStatus))             
        return VI_ERROR_PARAMETER2;
        
    if (windowStatus == VI_OFF) {
        Fmt (wrtBuffer,"%s<%s\r\n",s_showWindow[windowStatus]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
    else {
        Fmt (wrtBuffer,"%s<%s%f%s,%s%f%s,%s\r\n",
                        s_positionWindow[0],Xcenterposition,s_positionUnit[featuresPtr->span],
                        s_positionWindow[1],Xwidth,s_positionUnit[featuresPtr->span],
                        s_showWindow[windowStatus]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    } 
    
    return avr3132s_status;
}  

/*===========================================================================*/
/* Function: Read Measurement Window                                         */
/* Purpose:  returns the features and the status of the measurement window   */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_MeasurementWindow (ViSession instrID,ViPInt16 status,
                                                                 ViChar _VI_FAR message[],
                                                                 ViPReal64 Xposition,
                                                                 ViPReal64 Xwidth)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    avr3132s_ptrFeatures featuresPtr;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;

    /*reads the status of the measurement window*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    Fmt(message,"%s<%s",s_textONOFF[*status]);
    
    /* measurement window is not active -> activate it to measure its features */
    if (*status == 0) {  
        Fmt (Buffer,"%s<%s\r\n",s_showWindow[1]);   
        if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
    
    /*reads the position of the window along the X-axis*/
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[1]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f", Xposition)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    /*reads the width of the window along the X-axis*/ 
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[2]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f", Xwidth)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    /* measurement window is not active -> deactivate it */
    if (*status == 0) {  
        Fmt (Buffer,"%s<%s\r\n",s_showWindow[0]);   
        if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set Display Line                                                */           
/* Purpose:  Specifies the display line level and show or hide it            */
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_DisplayLine (ViSession instrID,ViBoolean showDL,
                                                          ViReal64 DLlevel)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    ViInt16 instrument; 

    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
        
    if (avr3132s_invalidViBooleanRange (showDL))             
        return VI_ERROR_PARAMETER2;
    
    instrument = featuresPtr->instrumentType;
    
    if (instrument == 3131) instrument = 0; else instrument = 1; 
    
    if (showDL == VI_ON) { /* activation */
        if (avr3132s_invalidViReal64Range (DLlevel,featuresPtr->DLlevelMin,
                                               featuresPtr->DLlevelMax))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s %f%s\r\n",s_showDL[showDL + 2*instrument],DLlevel,
                                        s_levelUnit[featuresPtr->refUnit]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
    else {  /* deactivation */
        Fmt (wrtBuffer,"%s<%s\r\n",s_showDL[showDL + 2*instrument]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    } 

    return avr3132s_status;                                          
}

/*===========================================================================*/
/* Function: Read Display Line                                               */
/* Purpose:  returns the position and the status of the Display Line         */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_DisplayLine (ViSession instrID, ViPInt16 status,
                                                            ViChar _VI_FAR message[],
                                                            ViPReal64 level)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    /* returns the display line status */
    
    Fmt (Buffer,"%s<%s\r\n",s_readDL[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    Fmt(message,"%s<%s",s_textONOFF[*status]);
    
    /* returns the display line level */
    
    if (*status==0) {
        /* activate the display line */
        Fmt (Buffer,"%s<%s\r\n",s_showDL[1]);   
        if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
    
    Fmt (Buffer,"%s<%s\r\n",s_readDL[1]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
        
    if (Scan(Buffer,"%s>%f", level)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    if (*status==0) {
        /* deactivate the display line */
        Fmt (Buffer,"%s<%s\r\n",s_showDL[0]);   
        if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
   
    return avr3132s_status;
}
    
/*===========================================================================*/
/* Function: Set Reference Line                                              */           
/* Purpose:  Specifies the reference line position and show or hide it       */
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_ReferenceLine (ViSession instrID,ViBoolean showRefLine,
                                                            ViReal64 refLineLevel)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    ViInt16 instrument;

    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;

    if (avr3132s_invalidViBooleanRange (showRefLine))             
        return VI_ERROR_PARAMETER2;
        
    instrument = featuresPtr->instrumentType;
    if (instrument == 3131) instrument = 0; else instrument = 1;
    
    if (showRefLine == VI_ON) { /* activation */
        Fmt (wrtBuffer,"%s<%s %f%s\r\n",s_showRefLine[showRefLine + 2*instrument],refLineLevel,
                                        s_levelUnit[featuresPtr->refUnit]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
    else {  /* deactivation */
        Fmt (wrtBuffer,"%s<%s\r\n",s_showRefLine[showRefLine + 2*instrument]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    } 

    return avr3132s_status;                                          
}   
    
/*===========================================================================*/
/* Function: Read Reference Line                                             */
/* Purpose:  returns the position and the status of the Display Line         */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_ReferenceLine(ViSession instrID, ViPInt16 status,
                                                             ViChar _VI_FAR message[],
                                                             ViPReal64 level)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    avr3132s_ptrFeatures featuresPtr;
    ViInt16 instrument;

    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
	
    instrument = featuresPtr->instrumentType;
    if (instrument == 3131) instrument = 0; else instrument = 1;

    /* returns the reference line status */
    
    Fmt (Buffer,"%s<%s\r\n",s_readRefLine[2*instrument]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
        
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
        
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }    
        
    Fmt(message,"%s<%s",s_textONOFF[*status]);
    
    /* returns the reference line level */
    
    if (*status==0) {
        /* activate the reference line */
        Fmt (Buffer,"%s<%s\r\n",s_showRefLine[1 + 2*instrument]);   
        if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
    
    Fmt (Buffer,"%s<%s\r\n",s_readRefLine[1 + 2*instrument]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
        
    if (Scan(Buffer,"%s>%f", level)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    if (*status==0) {
        /* deactivate the reference line */
        Fmt (Buffer,"%s<%s\r\n",s_showRefLine[2*instrument]);   
        if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set Limit Line                                                  */
/* Purpose:  Defines or clears a whole limit line table                      */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_LimitLines (ViSession instrID, ViBoolean limitLine,
                                       ViInt16 tblSize,
                                       ViReal64 *xTbl, ViReal64 *yTbl,
                                       ViReal64 sftX, ViReal64 sftY)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    ViUInt32 total = 0;
    ViInt16 i;
    ViString cmnd, unit;
    avr3132s_ptrFeatures featuresPtr;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    
    if (avr3132s_invalidViBooleanRange (limitLine))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViInt16Range (tblSize, 0, 51))             
    	return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViReal64Range (sftX, featuresPtr->centerLow, featuresPtr->centerHigh))             
        return VI_ERROR_PARAMETER6;    
    
    Fmt (Buffer,"%s<%s; %s",s_LLType[featuresPtr->span],s_deleteLL[limitLine]);   
    total += NumFmtdBytes();
    
    cmnd = s_setLL[limitLine];
    unit = s_positionUnit[featuresPtr->span];
    for (i=0; i<tblSize; i++)
    {
    	Fmt (Buffer,"%s[a]<; %s %f[e]%s,%f[e]", cmnd, *xTbl++, unit, *yTbl++);
        total += NumFmtdBytes();     
    }
    
    Fmt (Buffer, "%s[a]<; LIMSFT %f[e]%s; LIMASFT %f[e]", sftX, unit, sftY);
    total += NumFmtdBytes();
    
    if ((avr3132s_status = viWrite(instrID, Buffer, total, &retCount))<0)
        return avr3132s_status;
        
    return avr3132s_status; 
} 

/*===========================================================================*/
/* Function: Activate Limit Line                                             */
/* Purpose:  Show Limit Line 1 and/or Limit Line 2                           */
/*           Choose between absolute or relative along the two axis          */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Activate_LimitLine (ViSession instrID,ViBoolean LL1Status,
                                                             ViBoolean LL2Status,
                                                             ViInt16 XpositionMode,
                                                             ViInt16 YpositionMode,
                                                             ViBoolean passfail)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (avr3132s_invalidViBooleanRange (LL1Status))             
        return VI_ERROR_PARAMETER2;
        
    if (avr3132s_invalidViBooleanRange (LL2Status))             
        return VI_ERROR_PARAMETER3;
        
    if (avr3132s_invalidViInt16Range (XpositionMode,0,2))             
        return VI_ERROR_PARAMETER4;
    
    if (avr3132s_invalidViInt16Range (YpositionMode,0,2))             
        return VI_ERROR_PARAMETER5;
        
    if (avr3132s_invalidViBooleanRange (passfail))             
        return VI_ERROR_PARAMETER6;
        
        
    
    Fmt (wrtBuffer,"%s<%s,%s,%s,%s,%s\r\n",s_showLL1[LL1Status],s_showLL2[LL2Status],
                                           s_positionX[XpositionMode],s_positionY[YpositionMode],
                                           s_LLpassfail[passfail]);   
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    return avr3132s_status;                                          
}

/*===========================================================================*/
/* Function: Limit Line Judgment                                             */           
/* Purpose:  returns if the test with the limit line(s) is OK                */
/*===========================================================================*/    
ViStatus _VI_FUNC avr3132s_Read_LLjudgment (ViSession instrID, ViInt16 jdgOpr,
                                        ViInt16 *jdgStatus, ViChar judge[],
                                        ViInt16 *numFail,
                                        ViReal64 freqFail[], ViReal64 levelFail[]) 
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 i;
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_LLjudgement[jdgOpr]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
        
    if (jdgOpr==0) /* Judgement Result */
    {
    	if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        	return avr3132s_status;
        
    	if (Scan(Buffer,"%s>%d[b2]", jdgStatus) < 1) 
        	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        
    	Fmt(judge, "%s<%s", s_judgementText[*jdgStatus]); 
    }
    
    else /* Fail Point Up or Low */ 
    {
    	if ((avr3132s_status = viRead(instrID, Buffer, BUFFER_SIZE, &retCount))<0)
        	return avr3132s_status;
        	
        if (Scan (Buffer, "%s>%d[b2]", numFail) < 1)
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   		
   	   	for (i=0; i < *numFail; i++)
   	   	{
   	   		if ((avr3132s_status = viRead (instrID, Buffer, 51, &retCount)) < 0)
   	   			return avr3132s_status;
	   	
	   		if (Scan (Buffer, "%s>%f,%f", freqFail++, levelFail++) < 2)
   		   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   	}	
    }
    
    return avr3132s_status; 
}

/*===========================================================================*/
/* Function: Read Limit Line Status                                          */
/* Purpose:  returns the status of the Limit Line                            */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_LLstatus (ViSession instrID, ViBoolean parameter,
                                      ViPInt16 status, ViChar _VI_FAR message[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    
    if (avr3132s_invalidViBooleanRange (parameter))             
        return VI_ERROR_PARAMETER2; 
 
    Fmt (Buffer,"%s<%s\r\n",s_readLLstatus[parameter]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
     
    Fmt(message,"%s<%s",s_textONOFF[*status]); 
    
    return avr3132s_status; 
}        

/*===========================================================================*/
/* Function : Read Limit Line Options                                        */
/* Purpose : returns the options used with the limit lines                   */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_LLoptions (ViSession instrID, ViPInt16 XpositionMode,
                                       					  ViPInt16 YpositionMode)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    // returns the X position mode
    
    Fmt (Buffer,"%s<%s\r\n",s_readLLoptions[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
       
    if (Scan(Buffer,"%s>%d[b2]", XpositionMode)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
     
    // returns the Y position mode
    
    Fmt (Buffer,"%s<%s\r\n",s_readLLoptions[1]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
       
    if (Scan(Buffer,"%s>%d[b2]", YpositionMode)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    return avr3132s_status;
}  
    
    
/*===========================================================================*/
/* Function: Set Limit UP and LOW                                            */
/* Purpose:  Sets the upper and lower limits and shows or hides the level    */
/*           window                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_LimitUPandLOW (ViSession instrID,ViBoolean windowStatus,
                                                            ViReal64 lowerlimit,
                                                            ViReal64 upperlimit)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    avr3132s_ptrFeatures featuresPtr;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    
    if (avr3132s_invalidViBooleanRange (windowStatus))             
        return VI_ERROR_PARAMETER2;
        
    if (windowStatus == VI_OFF) {
        Fmt (wrtBuffer,"%s<%s\r\n",s_showLimit[windowStatus]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
    else {
        Fmt (wrtBuffer,"%s<%s%f%s,%s%f%s\r\n",
                        s_showLimit[2],lowerlimit,s_levelUnit[featuresPtr->refUnit],
                        s_showLimit[3],upperlimit,s_levelUnit[featuresPtr->refUnit]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    } 
    
    return avr3132s_status;
}  

/*===========================================================================*/
/* Function: Read Limit UP and LOW                                           */
/* Purpose:  returns the position and the status of the Level Window         */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_LimitUPandLOW (ViSession instrID, ViPInt16 status,
                                                              ViChar _VI_FAR message[],
                                                              ViPReal64 lowerLevel,
                                                              ViPReal64 upperLevel)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
	avr3132s_ptrFeatures featuresPtr;
	
    if ((avr3132s_status = viGetAttribute (instrID, VI_ATTR_USER_DATA, &featuresPtr)) < 0)
        return avr3132s_status;
    
    if (featuresPtr->instrumentType==3131)
    {
    	/* returns the level window status */
    	Fmt (Buffer,"%s<%s\r\n",s_readLimit[0]);   
    	if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        	return avr3132s_status;
        
   		if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        	return avr3132s_status; 
        
    	if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        	avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        	return avr3132s_status;
    	}    
   
    	Fmt(message,"%s<%s",s_textONOFF[*status]);
    
    	/* returns the lower level value */
    	Fmt (Buffer,"%s<%s\r\n",s_readLimit[1]);   
    	if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        	return avr3132s_status;
    
    	if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        	return avr3132s_status;
        
    	if (Scan(Buffer,"%s>%f", lowerLevel)!=1) { 
        	avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        	return avr3132s_status;
    	}
    
    	/* returns the upper level value */
    	Fmt (Buffer,"%s<%s\r\n",s_readLimit[2]);   
    	if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        	return avr3132s_status;
    
    	if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        	return avr3132s_status;
        
    	if (Scan(Buffer,"%s>%f", upperLevel)!=1) { 
        	avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
       		return avr3132s_status;
    	}
    }
    
    else
    	Fmt (message, "%s<Not R3131");
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read Limit UP and LOW judgment                                  */
/* Purpose:  returns the PASS/FAIL judgment                                  */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_LimitUPandLOW_Judgment (ViSession instrID, ViPInt16 result,
                                                                       ViChar _VI_FAR message[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
	
    if ((avr3132s_status = viGetAttribute (instrID, VI_ATTR_USER_DATA, &featuresPtr)) < 0)
        return avr3132s_status;
    
    if (featuresPtr->instrumentType==3131)
    {
    	Fmt (Buffer,"%s<%s\r\n",s_readLimit[3]);   
    	if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        	return avr3132s_status;
    
    	if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        	return avr3132s_status;
        
    	if (Scan(Buffer,"%s>%d[b2]", result)!=1) { 
        	avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        	return avr3132s_status;
    	}
    	
    	Fmt(message,"%s<%s",s_judgmentText[*result]); 
    }
    
    else
    	Fmt (message, "%s<Not R3131");
    
    return avr3132s_status; 
}

/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Write_InstrData (ViSession instrID, ViString writeBuffer)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (wrtBuffer,"%s<%s\r\n",writeBuffer);     
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status; 
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read Instr Data                                                 */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_InstrData (ViSession instrID, ViInt32 numBytes,
                                                          ViChar _VI_FAR readBuffer[],
                                                          ViPInt32 bytesRead)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    *bytesRead = 0L;
        
    if ((avr3132s_status = viRead (instrID, readBuffer, numBytes, bytesRead)) < 0)
        return avr3132s_status;
        
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_reset (ViSession instrSession)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    avr3132s_ptrFeatures featuresPtr;
    
    if ((avr3132s_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &featuresPtr)) < 0)
        return avr3132s_status;
    
    /*  Initialize the instrument to a known state.  */
    Fmt (wrtBuffer,"%s<%s\r\n",s_reset[0]);     
    if ((avr3132s_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
   
    if ((avr3132s_status = avr3132s_defaultInstrSetup (instrSession, featuresPtr->instrumentType)) < 0)  
        return avr3132s_status;
        
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_selfTest (ViSession instrID, ViPInt16 selfTestResult,
                                                    ViChar _VI_FAR selfTestMessage[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 index;
    
    Fmt (Buffer,"%s<%s\r\n",s_selfTest[0]);     
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
       
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
        
    if (Scan(Buffer,"%s>%d[b2]", selfTestResult)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    if (*selfTestResult!=0) {
    	index = (int)(log10(*selfTestResult)/log10(2));
    	Fmt (selfTestMessage, "%s<%s", s_selfTestMessage[index]);
    }
    else {
    	Fmt (selfTestMessage, "%s<%s", "Self Test OK"); 
    }	
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_errorQuery (ViSession instrID, ViPInt32 errorCode,
                                                      ViChar _VI_FAR errorMessage[])
{
    ViUInt32 retCnt = 0;
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_errorQuery[0]);     
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
       
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b4]", errorCode)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    avr3132s_errorMessage (instrID, *errorCode, errorMessage);
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_errorMessage (ViSession instrID, ViStatus errorCode,
                                                        ViChar _VI_FAR errorMessage[])
 
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViInt16 i;
    
    static avr3132s_tStringValPair statusDescArray[] = 
    {
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_ERROR_PARAMETER1,        "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,        "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,        "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,        "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,        "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,        "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,        "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,        "ERROR: Parameter 8 out of range"},
        {VI_ERROR_INSTR_PARAMETER9 , "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10, "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11, "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12, "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13, "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14, "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15, "ERROR: Parameter 15 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,    "ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE,     "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,  "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE, "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_MARKERINACTIVE,   "ERROR: the chosen multi-marker is inactive"},
        {VI_ERROR_SCANNING_FREQ_TABLE,   "ERROR: there was an error scanning the Correction FrequencyTable"},
        {VI_ERROR_SCANNING_LEV_TABLE,   "ERROR: there was an error scanning the Correction Level Table"},
        {1, "ERROR : Span is set to 0 Hz - Please change span"},
        {2, "ERROR : Scale is Linear Mode - Please select dB/div scale"},
        {3, "ERROR : QP detector is active - Please change to Normal"},
        {4, "ERROR : Antenna correction is ON"},
        {5, "ERROR : Scale is not 10 dB/div"},
        {6, "ERROR : Delta marker is not active"},
        {7, "ERROR : Blank mode is selected - Please change to Write mode"},
        {8, "ERROR : Calculated power is out of range"},
        {9, "ERROR : No peak is detected"},
        {10, "ERROR : Parameter is set over the scale"},
        {11, "ERROR : Ant Corr data is not save to RAM - Please select device Floppy Disk"},
        {100, "ERROR : IF STEP AMP : calibration failure"},
        {101, "ERROR : LOG LINEARITY : calibration failure"},
        {102, "ERROR : TOTAL GAIN : calibration failure"},
        {103, "ERROR : RBW SWITCHING : calibration failure"},
        {104, "ERROR : AMPTD MAG : calibration failure"},
        {105, "ERROR : Cal data is not enough - Please execute Cal All"},
        {106, "ERROR : Calibration signal is not detected"},
        {150, "ERROR : Self Test Failure - Please Report to qualified service person"},
        {151, "ERROR : Self Test Failure - Please Report to qualified service person"},
        {152, "ERROR : Self Test Failure - Please Report to qualified service person"},
        {153, "ERROR : Self Test Failure - Please Report to qualified service person"},
        {154, "ERROR : Self Test Failure - Please Report to qualified service person"},
        {155, "ERROR : Self Test Failure - Please Report to qualified service person"},
        {200, "ERROR : Illegal Parameters"},
        {201, "ERROR : Software version unmatched"},
        {202, "ERROR : Cannot format a device"},
        {203, "ERROR : File or register empty"},
        {204, "ERROR : Trace buffer full"},
        {205, "ERROR : device not ready"},
        {206, "ERROR : Read error"},
        {207, "ERROR : File not found"},
        {208, "ERROR : Invalid BPB - Please format a disk"},
        {209, "ERROR : Can't delete a file (read-only file)"},
        {210, "ERROR : Media changed"},
        {211, "ERROR : No disk space"},
        {212, "ERROR : Read Only file"},
        {213, "ERROR : Read only media"},
        {214, "ERROR : Root directory full"},
        {215, "ERROR : Invalid boot sector signature"},
        {216, "ERROR : CRC error"},
        {217, "ERROR : Invalid disk geometry"},
        {218, "ERROR : File Number is over 999 - Please reset file counter"},
        {300, "ERROR : Printer is not Ready"},
        {301, "ERROR : Printer Cable Problem"},
        {302, "ERROR : Printer is not active"},
        {VI_NULL, VI_NULL}
     };

    avr3132s_status = viStatusDesc (instrID, errorCode, errorMessage);
    
    if (avr3132s_status == VI_WARN_UNKNOWN_STATUS) {
        for (i=0; statusDescArray[i].stringName; i++) {
            if (statusDescArray[i].stringVal == errorCode) {
                Fmt (errorMessage, "%s<%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
            }
        }
        Fmt (errorMessage, "%s<Unknown Error 0x%x[uw8p0]", errorCode); 
        return (VI_WARN_UNKNOWN_STATUS);
    }
    
    avr3132s_status = VI_SUCCESS;
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_revisionQuery (ViSession instrID, ViChar _VI_FAR driverRev[],
                                                         ViChar _VI_FAR instrRev[])
{
    ViUInt32 retCount = 0;
    ViStatus avr3132s_status = VI_SUCCESS;
    ViChar Buffer[BUFFER_SIZE];
    
    Fmt (Buffer,"%s<%s\r\n",s_init[1]);     
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
   
    if ((avr3132s_status = viRead (instrID, Buffer, BUFFER_SIZE, &retCount)) < 0)
        return avr3132s_status;
    if (Scan (Buffer, "%s>%s[t44xd]%s[t44xd]%s[t44xd]%s[t13]", instrRev) != 4) {
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    Fmt (driverRev, "%s<%s", AVR3132S_REVISION);
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Calibration                                                     */
/* Purpose:  Set calibration to element                                      */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Calibration (ViSession instrID, ViInt16 element,
                                                       ViReal64 level,
                                                       ViBoolean calCorrection,
                                                       ViBoolean freqCorrection)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    ViUInt32 timeout;
    ViInt16 instrument;

    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    
    if (avr3132s_invalidViInt16Range (element,0,6))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViReal64Range (level,featuresPtr->calLevelMin, featuresPtr->calLevelMax))          
        return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViBooleanRange (calCorrection))             
        return VI_ERROR_PARAMETER4;
    if (avr3132s_invalidViBooleanRange (freqCorrection))             
        return VI_ERROR_PARAMETER5;
    
    instrument = featuresPtr->instrumentType;
    if (instrument == 3131) instrument = 0; else instrument = 1;    
        
    /* stores the value of the timeout in variable timeout and then disables the timeout */
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_TMO_VALUE,&timeout)) < 0)
        return avr3132s_status;
    if ((avr3132s_status = viSetAttribute (instrID, VI_ATTR_TMO_VALUE, 600000)) < 0)
        return avr3132s_status;
       
    /* element to calibrate*/  
    Fmt (wrtBuffer,"%s<%s %fDB,%s\r\n",s_calLevel[0],level,s_calibration[element + 7*instrument]);     
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;     

    /* calibration and frequency correction */ 
    Fmt (wrtBuffer,"%s<%s,%s\r\n",s_freqCorrection[freqCorrection],
                                  s_calCorrection[calCorrection]);     
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
    
    /* disables the calibration's signal */
    if (instrument == 0) {
    	Fmt (wrtBuffer,"%s<%s\r\n",s_calLevel[1]);     
    	if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        	return avr3132s_status;  
    }    	
   
    /*  enables the timeout and sets its value to its last value */  
    if ((avr3132s_status = viSetAttribute (instrID, VI_ATTR_TMO_VALUE, timeout)) < 0)
        return avr3132s_status;
        
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Execute Auto Tune                                               */
/* Purpose:  This function allows to display the maximum peak of the signal  */
/*           at the middle of the screen                                     */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Execute_AutoTuning (ViSession instrSession)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    avr3132s_ptrFeatures featuresPtr;
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_autotune[0]);     
    if ((avr3132s_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
   
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Display label                                                   */
/* Purpose:  selects label                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Display_Label (ViSession instrID, ViBoolean showLabel,
                                                         ViString label)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViBooleanRange (showLabel))             
        return VI_ERROR_PARAMETER2;
       
    if (showLabel == VI_ON) {
        if (CompareStrings(label,0,"",0,0)==0) return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s%s/\r\n",s_label[showLabel],label);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status; 
    }
    else {
        Fmt (wrtBuffer,"%s<%s\r\n",s_label[showLabel]);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;
    }
    
    return avr3132s_status;   
}

/*===========================================================================*/
/* Function : Read Status Byte                                               */
/* purpose : controls the status byte register                               */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_StatusByte (ViSession instrID, ViPInt16 statusByte)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViUInt16 status = 0;
    ViByte Buffer[BUFFER_SIZE];
   
    avr3132s_status = viReadSTB (instrID, &status);
    * statusByte = (ViInt16) status;
  
    return avr3132s_status;
}

/*===========================================================================*/
/* Function : Clear Status Byte                                              */
/* purpose : clears the status byte register                                 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Clear_StatusByte (ViSession instrID)
{
 
    ViStatus avr3132s_status = VI_SUCCESS;
    ViUInt32 retCount = 0;
    ViByte Buffer[BUFFER_SIZE];
    
    Fmt (Buffer,"%s<%s\r\n",s_clearStatusByte[0]);     
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
       
    return avr3132s_status;
   
}

/*===========================================================================*/
/* Function : Convert Frequency                                              */
/* Purpose : converts the frequency value returned in Hz by the analyzer     */
/*           in the value with the best unit (Hz, kHz, MHz or GHz)           */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Convert_Frequency (ViSession instrID,ViReal64 inputValue,
                                                            ViPReal64 outputValue,
                                                            ViChar _VI_FAR frequencyUnit[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViReal64 denom[]={1,1000,1000000,1000000000},interim[3];
    ViInt16 i;
    
    for (i=0;i<3;i++) {
        interim[i] = inputValue / denom[i+1];
    }
    
    i=0; 
    while ( (i<3)&&(fabs(interim[i])>=1) ) {
        i++;
    }
     
    *outputValue = inputValue / denom[i];
    Fmt(frequencyUnit,"%s<%s",s_freqUnit[i]); 
    return avr3132s_status;
} 
 
/*===========================================================================*/
/* Function : Convert Time                                                   */
/* Purpose : converts the time value returned in second(s) by the analyzer   */
/*           in the value with the best unit (ns, us, ms or s)               */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Convert_Time (ViSession instrID, ViReal64 inputValue,
                                                        ViPReal64 outputValue,
                                                        ViChar _VI_FAR timeUnit[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViReal64 compare[]={1000000000,1000000,1000,1};
    ViInt16 i;
    
    i=0;
    while ((i<=2)&&(fabs(inputValue)*compare[i]>=1000)) {
        i++;
    }
     
    *outputValue = inputValue*compare[i];
    Fmt(timeUnit,"%s<%s",s_timeUnit[i]);
    return avr3132s_status;
 
}  

/*===========================================================================*/
/* Function: Execute Power Measurement                                       */
/* Purpose: execute different power measurement                              */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Execute_PowerMeasurement (ViSession instrID,
												 	 ViInt16 powerMeasurement, ViInt16 averagingCount)
{                                                               
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
        
    if (avr3132s_invalidViInt16Range (powerMeasurement, 0, 4))             
        return VI_ERROR_PARAMETER2;
    if (featuresPtr->instrumentType!=3131 && powerMeasurement==3)
    	return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViInt16Range (averagingCount,featuresPtr->averageCountMini - 1,
                                                     featuresPtr->averageCountMaxi - 1))          
        return VI_ERROR_PARAMETER3;
        	
    Fmt (Buffer, "%s<PWTM %d; %s", averagingCount, s_powerMeas[powerMeasurement]);
        
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
    	return avr3132s_status; 
    
    return avr3132s_status;        
}    

/*===========================================================================*/
/* Function: Read Power Measurement                                          */
/* Purpose: returns the value of the power measured                          */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_PowerMeasurement (ViSession instrID, ViInt16 power,
                                                                 ViPReal64 powerValue,
                                                                 ViPInt16 averagingCount)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    
    if (avr3132s_invalidViInt16Range (power, 0, 3))             
      	return VI_ERROR_PARAMETER2;    
    if (featuresPtr->instrumentType!=3131 && power==3)
       	return VI_ERROR_PARAMETER2;
       	
    
    /* returns the value of the power */
    Fmt (Buffer,"%s<%s\r\n",s_readPowerMeas[power]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f", powerValue)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    /* returns the value of the averaging count */
    if ((avr3132s_status = viWrite(instrID, "PWTM?", 5,&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", averagingCount)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    return avr3132s_status;
}    

/*===========================================================================*/
/* Function: Execute ACP Measurement (R3131)                                 */
/* Purpose:  This function executes ACP measurement                          */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Execute_ACP_R3131 (ViSession instrID,
                                              ViReal64 bandWidth, ViReal64 chnlSpc1,
                                              ViBoolean selChnlSpc2, ViReal64 chnlSpc2,
                                              ViBoolean graphACP, ViBoolean measACP)
{
 	ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount = 0;
    ViUInt32 total = 0;
    
    if (avr3132s_invalidViReal64Range (bandWidth, 0.0, 3.0e9))
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViReal64Range (chnlSpc1, 0.0, 3.0e9))
        return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViBooleanRange (selChnlSpc2))
        return VI_ERROR_PARAMETER4;
    if (avr3132s_invalidViReal64Range (chnlSpc2, 0.0, 3.0e9))
        return VI_ERROR_PARAMETER5;
    if (avr3132s_invalidViBooleanRange (graphACP))
        return VI_ERROR_PARAMETER6;
    if (avr3132s_invalidViBooleanRange (measACP))
        return VI_ERROR_PARAMETER7;
        
    Fmt (wrtBuffer, "%s<ADBS %fHZ; ADCH %fHZ", bandWidth, chnlSpc1);
    total += NumFmtdBytes();
    
    if (selChnlSpc2==VI_ON)
    	Fmt (wrtBuffer, "%s[a]<; ADCHSC %fHZ", chnlSpc2);
    else
    	Fmt (wrtBuffer, "%s[a]<; ADCHSC OFF");
    total += NumFmtdBytes();
    
    if (graphACP==VI_OFF && measACP==VI_OFF)
    	Fmt (wrtBuffer, "%s[a]<; ADG OFF; ACP OFF");
    else if (measACP==VI_ON)
    	Fmt (wrtBuffer, "%s[a]<; ACP");
    else
    	Fmt (wrtBuffer, "%s[a]<; ADG");
    total += NumFmtdBytes();
    
    if ((avr3132s_status = viWrite(instrID, wrtBuffer, total, &retCount)) < 0)
    	return avr3132s_status; 
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Execute ACP Measurement (R3132s)                                */
/* Purpose:  This function executes ACP measurement                          */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Execute_ACP_R3132s (ViSession instrID,
                                  			   ViReal64 bandWidth, ViInt16 tblSize,
                                           	   ViReal64 tblCS[], ViReal64 tblBS[],
                                           	   ViBoolean rtNyqFltr, ViReal64 symbolRate,
                                           	   ViReal64 rloffFctr, ViBoolean bsWindow,
                                           	   ViInt16 screen, ViBoolean graph,
                                           	   ViBoolean exeMeas)
{
 	ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount = 0;
    ViUInt32 total = 0;
    ViInt32  n;
    ViReal64 *ptCS, *ptBS;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    if (avr3132s_invalidViReal64Range (bandWidth, featuresPtr->centerLow, featuresPtr->centerHigh))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViInt16Range (tblSize, 0, 5))             
        return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViBooleanRange (rtNyqFltr))             
        return VI_ERROR_PARAMETER6;
    if (avr3132s_invalidViReal64Range (symbolRate, 1.0, 1.0e9))             
        return VI_ERROR_PARAMETER7;
    if (avr3132s_invalidViReal64Range (rloffFctr, 0.01, 0.99))             
        return VI_ERROR_PARAMETER8;
    if (avr3132s_invalidViBooleanRange (bsWindow))             
        return VI_ERROR_INSTR_PARAMETER9;
    if (avr3132s_invalidViInt16Range (screen, 0, 2))             
        return VI_ERROR_INSTR_PARAMETER10;
    if (avr3132s_invalidViBooleanRange (graph))             
        return VI_ERROR_INSTR_PARAMETER11;
    if (avr3132s_invalidViBooleanRange (exeMeas))             
        return VI_ERROR_INSTR_PARAMETER12;
    
    n = tblSize;
    ptCS = tblCS;
    ptBS = tblBS;
    while (--n>=0)
    {
    	if (avr3132s_invalidViReal64Range(*ptCS++, 0.0, 1000.0e9))             
        	return VI_ERROR_PARAMETER4;
        if (avr3132s_invalidViReal64Range(*ptBS++, 0.0, 1000.0e9))             
        	return VI_ERROR_PARAMETER5;	
    }
    
    Fmt (wrtBuffer, "%s<ACP OFF; CARRBS %fHZ; CSBSDEL", bandWidth);
    total += NumFmtdBytes();
    
    n = tblSize;
    ptCS = tblCS;
    ptBS = tblBS;
    while (--n>=0)
    {
    	Fmt (wrtBuffer, "%s[a]<; CSBSIN %fHZ,%fHZ", *ptCS++, *ptBS++);
    	total += NumFmtdBytes();
    }

	Fmt (wrtBuffer, "%s[a]<; SYMRT %fHZ; RFACT %f; %s; %s", symbolRate, rloffFctr,
    	 s_rootNyqFilter[rtNyqFltr], s_ACPScreen[screen]);
    total += NumFmtdBytes();
    
    if (exeMeas==VI_ON)
    {
    	if (screen!=1 && graph==VI_ON)
    		Fmt (wrtBuffer, "%s[a]<; ACP ON; ADG ON");
    	else
    		Fmt (wrtBuffer, "%s[a]<;ACP ON; ADG OFF");
    }
    else
    	Fmt (wrtBuffer, "%s[a]<; ACP OFF"); 
    total += NumFmtdBytes();
    
    Fmt (wrtBuffer, "%s[a]<; %s", s_BSWindow[bsWindow]);
    total += NumFmtdBytes();

    if ((avr3132s_status = viWrite(instrID, wrtBuffer, total, &retCount))<0)
        return avr3132s_status;
    
    return avr3132s_status;    
}

/*===========================================================================*/
/* Function: Read ACP Measurement (R3131)                                    */
/* Purpose:  This function returns the values of the ACP measured.           */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_ACP_R3131 (ViSession instrID, ViBoolean selChnlSpc2,
                                           ViReal64 *pwrLwrChnl1, ViReal64 *pwrUpChnl1,
                                           ViReal64 *pwrLwrChnl2, ViReal64 *pwrUpChnl2)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    
    if (avr3132s_invalidViBooleanRange (selChnlSpc2))             
        return VI_ERROR_PARAMETER2;
        
    if (selChnlSpc2==VI_ON)
    {
    	if ((avr3132s_status = viWrite(instrID, "ACPSC?", 6, &retCount))<0)
       		return avr3132s_status;
    	if ((avr3132s_status = viRead (instrID, Buffer, 103, &retCount)) < 0)
        	return avr3132s_status;
    	if (Scan(Buffer,"%s>%f,%f,%f,%f", pwrLwrChnl1, pwrUpChnl1,
    			 pwrLwrChnl2, pwrUpChnl2) < 4) 
        	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    }
    
    else
    {
    	if ((avr3132s_status = viWrite(instrID, "ACP?", 4, &retCount))<0)
       		return avr3132s_status;
    	if ((avr3132s_status = viRead (instrID, Buffer, 51, &retCount)) < 0)
        	return avr3132s_status;
    	if (Scan(Buffer,"%s>%f,%f", pwrLwrChnl1, pwrUpChnl1) < 2) 
        	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        	
        *pwrLwrChnl2 = 0.0;
        *pwrUpChnl2  = 0.0;
    }
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read ACP Measurement (R3132s)                                   */
/* Purpose:  This function returns the values of the ACP measured.           */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_ACP_R3132s (ViSession instrID,
                                            ViReal64 *refPwr, ViInt16 *numData,
                                            ViReal64 lwrFreq[], ViReal64 levelLwrFreq[],
                                            ViReal64 upFreq[], ViReal64 levelUpFreq[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    ViInt32 n;

    if ((avr3132s_status = viWrite(instrID, "ACPREF?", 7, &retCount))<0)
        return avr3132s_status;
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f", refPwr) < 1) 
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avr3132s_status = viWrite(instrID, "ACP?", 4, &retCount))<0)
        return avr3132s_status;
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", numData) < 1) 
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        
    n = *numData;
    while (--n>=0)
    {
    	if ((avr3132s_status = viRead (instrID, Buffer, 103, &retCount)) < 0)
        	return avr3132s_status;
        if (Scan(Buffer,"%s>%f,%f,%f,%f", lwrFreq++, levelLwrFreq++,
        	upFreq++, levelUpFreq++) < 4) 
        	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;	
    }
        
    return avr3132s_status;    
}

/*===========================================================================*/
/* Function: Execute OBW Measurement                                         */
/* Purpose:  This function executes OBW measurement                          */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Execute_OBWMeasurement (ViSession instrID,
                                           		   ViReal64 valOBW, ViBoolean exeOBW)
{
	ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;

    if (avr3132s_invalidViReal64Range (valOBW, 10.0, 99.8))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViBooleanRange (exeOBW))             
        return VI_ERROR_PARAMETER3;
        
    Fmt (wrtBuffer, "%s<OBWPER %f; %s", valOBW, s_exeOBW[exeOBW]);
    
    if ((avr3132s_status = viWrite(instrID, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
    	return avr3132s_status;
    	
    return avr3132s_status;	
}

/*===========================================================================*/
/* Function: Read OBW Measurement                                            */
/* Purpose:  This function returns the value of the OBW measured.            */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_OBWMeasurement (ViSession instrID, ViReal64 *rateOBW, 
                                            ViReal64 *cntrFreq, ViReal64 *freqOBW)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    
    if ((avr3132s_status = viWrite(instrID, "OBWPER?", 7, &retCount))<0)
        return avr3132s_status;
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f", rateOBW) < 1) 
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        
    if ((avr3132s_status = viWrite(instrID, "OBW?", 4, &retCount))<0)
        return avr3132s_status;
    if ((avr3132s_status = viRead (instrID, Buffer, 51, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f,%f", cntrFreq, freqOBW) < 2) 
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;    
        
    return avr3132s_status;    
}

/*===========================================================================*/
/* Function: Set Sound Parameters                                            */
/* Purpose:  This function sets and activates the sound mode parameters.     */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Set_Sound (ViSession instrID, ViInt16 soundMode,
                                  ViInt16 volume, ViReal64 pauseTime,
                                  ViBoolean sqlchMode, ViReal64 sqlchValue)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViInt16Range (soundMode, 0, 3))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViInt16Range (volume, 1, 16))             
        return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViReal64Range (pauseTime, 0.1, 800))             
        return VI_ERROR_PARAMETER4;
    if (avr3132s_invalidViBooleanRange (sqlchMode))             
        return VI_ERROR_PARAMETER5;
    if (avr3132s_invalidViReal64Range (sqlchValue, -80.0, 0.0))             
        return VI_ERROR_PARAMETER6;
        
    Fmt (wrtBuffer, "%s<PU %fSC; SQE %f; %s; SDV %d, %s", pauseTime, sqlchValue,
    	 s_squelchMode[sqlchMode], volume, s_soundMode[soundMode]);
    	 
    if ((avr3132s_status = viWrite(instrID, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
    	return avr3132s_status;
    	
    return avr3132s_status;	 
}

/*===========================================================================*/
/* Function: Execute Spectrum Mask Measurement                               */
/* Purpose:  This function executes spectrum mask measurement                */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Execute_SpectrumMask (ViSession instrID,
                                                 ViReal64 bandWidth, ViInt16 tblSize,
                                                 ViReal64 tblCS[], ViReal64 tblBS[],
                                                 ViBoolean rtNyqFltr, ViReal64 symbolRate,
                                                 ViReal64 rloffFctr, 
                                                 ViBoolean dispMode, ViBoolean windowBS,
                                                 ViBoolean exeMeas)
{
 	ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount = 0;
    ViUInt32 total = 0;
    ViInt32  n;
    ViReal64 *ptCS, *ptBS;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    if (avr3132s_invalidViReal64Range (bandWidth, featuresPtr->centerLow, featuresPtr->centerHigh))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViInt16Range (tblSize, 0, 5))             
        return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViBooleanRange (rtNyqFltr))             
        return VI_ERROR_PARAMETER6;
    if (avr3132s_invalidViReal64Range (symbolRate, 1.0, 1.0e9))             
        return VI_ERROR_PARAMETER7;
    if (avr3132s_invalidViReal64Range (rloffFctr, 0.01, 0.99))             
        return VI_ERROR_PARAMETER8;
    if (avr3132s_invalidViBooleanRange (dispMode))             
        return VI_ERROR_INSTR_PARAMETER9;
    if (avr3132s_invalidViBooleanRange (windowBS))             
        return VI_ERROR_INSTR_PARAMETER10;
    if (avr3132s_invalidViBooleanRange (exeMeas))             
        return VI_ERROR_INSTR_PARAMETER11;
    
    n = tblSize;
    ptCS = tblCS;
    ptBS = tblBS;
    while (--n>=0)
    {
    	if (avr3132s_invalidViReal64Range(*ptCS++, 0.0, 1000.0e9))             
        	return VI_ERROR_PARAMETER4;
        if (avr3132s_invalidViReal64Range(*ptBS++, 0.0, 1000.0e9))             
        	return VI_ERROR_PARAMETER5;	
    }
    
    Fmt (wrtBuffer, "%s<ACP OFF; CARRBS %fHZ; CSBSDEL", bandWidth);
    total += NumFmtdBytes();
    
    n = tblSize;
    ptCS = tblCS;
    ptBS = tblBS;
    while (--n>=0)
    {
    	Fmt (wrtBuffer, "%s[a]<; CSBSIN %fHZ,%fHZ", *ptCS++, *ptBS++);
    	total += NumFmtdBytes();
    }

	Fmt (wrtBuffer, "%s[a]<; SYMRT %fHZ; RFACT %f; %s; %s; %s; %s", symbolRate, rloffFctr,
    	 s_rootNyqFilter[rtNyqFltr], s_dispMdSpecMask[dispMode], s_BSWindow[windowBS], s_exeSpecMask[exeMeas]);
    total += NumFmtdBytes();
    
    if ((avr3132s_status = viWrite(instrID, wrtBuffer, total, &retCount)) < 0)
    	return avr3132s_status;
    	
    return avr3132s_status;	 
}

/*===========================================================================*/
/* Function: Execute Spectrum Mask Measurement                               */
/* Purpose:  This function executes spectrum mask measurement                */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Execute_Spurious (ViSession instrID, ViBoolean msMode,
                                             ViInt16 tableNo, ViInt16 tableSize,
                                             ViReal64 strFreq[], ViReal64 stpFreq[],
                                             ViReal64 cntrFreq[], ViReal64 valRBW[],
                                             ViReal64 swpTime[], ViReal64 lmtLevel[],
                                             ViInt16 sweepCount, ViBoolean passJudge,
                                             ViBoolean exeMeas)
{
 	ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount = 0;
    ViUInt32 total = 0;
    ViInt32  n;
    ViReal64 *pt_str, *pt_stp, *pt_cntr, *pt_RBW, *pt_swp, *pt_lmt;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    if (avr3132s_invalidViBooleanRange (msMode))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViInt16Range (tableNo, 1, 3))             
        return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViInt16Range (tableSize, 0, 15))             
        return VI_ERROR_PARAMETER4;
    if (avr3132s_invalidViInt16Range (sweepCount, 1, 999))             
        return VI_ERROR_INSTR_PARAMETER11;
    if (avr3132s_invalidViBooleanRange (passJudge))             
        return VI_ERROR_INSTR_PARAMETER12;
    if (avr3132s_invalidViBooleanRange (exeMeas))             
        return VI_ERROR_INSTR_PARAMETER13;
        
    n = tableSize;
    pt_str  = strFreq;
    pt_stp  = stpFreq;
    pt_cntr = cntrFreq;
    pt_RBW  = valRBW;
    pt_swp  = swpTime;
    pt_lmt  = lmtLevel;
    while (--n>=0)
    {
    	if (avr3132s_invalidViReal64Range (*pt_str++, 0.0, featuresPtr->startHigh))             
        	return VI_ERROR_PARAMETER5;
        if (avr3132s_invalidViReal64Range (*pt_stp++, 0.0, featuresPtr->stopHigh))             
        	return VI_ERROR_PARAMETER6;
        if (avr3132s_invalidViReal64Range (*pt_cntr++, 0.0, featuresPtr->centerHigh))             
        	return VI_ERROR_PARAMETER7;
        if (avr3132s_invalidViReal64Range (*pt_RBW++, 1.0e3, 3.0e6))             
        	return VI_ERROR_PARAMETER8;
        if (avr3132s_invalidViReal64Range (*pt_swp++, 10.0e-3, 1000.0))             
        	return VI_ERROR_INSTR_PARAMETER9;
    }
    
    n = tableSize;
    pt_str  = strFreq;
    pt_stp  = stpFreq;
    pt_cntr = cntrFreq;
    pt_RBW  = valRBW;
    pt_swp  = swpTime;
    pt_lmt  = lmtLevel;
    
    if (msMode==0)  /* Spurious Time */
    {
    	Fmt (Buffer, "%s<SPRTBL %d; SPRTIN %fHZ,%fHZ,%f[e]SC,%f; SPURI TIME; SPURI OFF",
    		 tableNo, *pt_cntr, *pt_RBW, *pt_swp, *pt_lmt);
    	if ((avr3132s_status = viWrite(instrID, Buffer, NumFmtdBytes(), &retCount)) < 0)
    		return avr3132s_status;	 
    	
    	Fmt (Buffer, "%s<SPRDEL");
    	total += NumFmtdBytes();
    	while (--n>=0)
    	{
    		Fmt (Buffer, "%s[a]<; SPRTIN %fHZ,%fHZ,%f[e]SC,%f",
    			 *pt_cntr++, *pt_RBW++, *pt_swp++, *pt_lmt++);
    		total += NumFmtdBytes();	 
    	}
    }
    
    else
    {
    	Fmt (Buffer, "%s<SPRTBL %d; SPRIN %fHZ,%fHZ,%fHZ,%f[e]SC,%f; SPURI FREQ; SPURI OFF",
    		 tableNo, *pt_str, *pt_stp, *pt_RBW, *pt_swp, *pt_lmt);
    	if ((avr3132s_status = viWrite(instrID, Buffer, NumFmtdBytes(), &retCount)) < 0)
    		return avr3132s_status;	 
    	
    	Fmt (Buffer, "%s<SPRDEL");
    	total += NumFmtdBytes();
    	while (--n>=0)
    	{
    		Fmt (Buffer, "%s[a]<; SPRIN %fHZ,%fHZ,%fHZ,%f[e]SC,%f",
    			 *pt_str++, *pt_stp++, *pt_RBW++, *pt_swp++, *pt_lmt++);
    		total += NumFmtdBytes();	 
    	}
    }
    
    if (sweepCount==1)
    	Fmt (Buffer, "%s[a]<; SPRCNT OFF");
    else
    	Fmt (Buffer, "%s[a]<; SPRCNT %d", sweepCount);
    total += NumFmtdBytes();
    
    Fmt (Buffer, "%s[a]<; %s", s_passJudge[passJudge]);
    total += NumFmtdBytes();
    
    if (msMode==0)  /* Spurious Time */
    {
    	if (exeMeas==1)  /* Measurement ON */
    		Fmt (Buffer, "%s[a]<; SPURI TIME");
    	else			 /* Measurement OFF */
    		Fmt (Buffer, "%s[a]<; SPURI OFF");
    	total += NumFmtdBytes();	
    }
    
    else			/* Spurious Frequency */ 
    {
    	if (exeMeas==1)  /* Measurement ON */
    		Fmt (Buffer, "%s[a]<; SPURI FREQ");
    	else			 /* Measurement OFF */
    		Fmt (Buffer, "%s[a]<; SPURI OFF");
    	total += NumFmtdBytes();
    }
    
    if ((avr3132s_status = viWrite(instrID, Buffer, total, &retCount)) < 0)
    	return avr3132s_status;
    	
    return avr3132s_status;	 
}

/*===========================================================================*/
/* Function: Execute Noise/Hz Parameters                                     */
/* Purpose:  This function sets the parameters of the Noise/Hz measurement.  */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Execute_NoiseHz (ViSession instrID,
                                    		ViInt16 mode, ViReal64 freq)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViInt16Range (mode, 0, 3))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViReal64Range (freq, 1.0, 100.0e6))             
        return VI_ERROR_PARAMETER3;
        
    Fmt (wrtBuffer, "%s<NI %fHZ; %s", freq, s_noiseHzMode[mode]);
    
    if ((avr3132s_status = viWrite(instrID, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
    	return avr3132s_status;
    	
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read Spectrum Mask Measurement                                  */
/* Purpose:  This function returns the values of the spectrum mask measured. */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_SpectrumMask (ViSession instrID,
                                              ViReal64 *refPwr, ViInt16 *numData,
                                              ViReal64 lwrFreq[], ViReal64 levelLwrFreq[],
                                              ViReal64 upFreq[],  ViReal64 levelUpFreq[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    ViInt32 n;

    if ((avr3132s_status = viWrite(instrID, "SPM?", 4, &retCount))<0)
        return avr3132s_status;
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f", refPwr) < 1) 
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", numData) < 1) 
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        
    n = *numData;
    while (--n>=0)
    {
    	if ((avr3132s_status = viRead (instrID, Buffer, 103, &retCount)) < 0)
        	return avr3132s_status;
        if (Scan(Buffer,"%s>%f,%f,%f,%f", lwrFreq++, levelLwrFreq++,
        	upFreq++, levelUpFreq++) < 4) 
        	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;	
    }
        
    return avr3132s_status;    
}

/*===========================================================================*/
/* Function: Read Spurious Measurement		                                 */
/* Purpose:  This function returns the results of the spurious measurement.  */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_Spurious (ViSession instrID,
                                          ViInt16 *msPoints, ViInt16 numSprSig[],
                                          ViReal64 sprFreq[], ViReal64 sprLvl[],
                                          ViInt16 judge[])
{
	ViStatus avr3132s_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    ViInt32 m, n;

    if ((avr3132s_status = viWrite(instrID, "SPURI?", 6, &retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%d[b2]", msPoints) < 1) 
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        
    n = *msPoints;
    while (--n>=0)
    {
    	if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        	return avr3132s_status;
    	if (Scan(Buffer,"%s>%d[b2]", numSprSig) < 1) 
        	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    	m = *numSprSig++;
    	while (--m>=0)
    	{
    		if ((avr3132s_status = viRead (instrID, Buffer, 62, &retCount)) < 0)
        		return avr3132s_status;
        	if (Scan(Buffer,"%s>%f,%f,%d[b2]", sprFreq++, sprLvl++, judge++) < 3) 
        		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }		
    }
        
    return avr3132s_status;     
}

/*===========================================================================*/
/* Function: Read Noise/Hz Measurement                                       */
/* Purpose:  This function returns the results of the Noise/Hz measurement.  */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_NoiseHz (ViSession instrID, ViReal64 *power)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if ((avr3132s_status = viWrite(instrID, "NIRES?", 6, &retCount))<0)
        return avr3132s_status;
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f", power) < 1) 
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        
    return avr3132s_status;    
}

/*===========================================================================*/
/* Function: save a file                                                     */
/* Purpose:  stores the data in the internal memory or on a floppy disk      */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Save_File (ViSession instrID, ViBoolean drive,
                                                     ViInt16 registerNumber,
                                                     ViString fileName)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViBooleanRange (drive))             
        return VI_ERROR_PARAMETER2;
        
    if (drive == VI_ON) { /* stores data on a floppy disk */
        if (CompareStrings(fileName,0,"",0,0)==0) return VI_ERROR_PARAMETER4;
        Fmt (wrtBuffer,"%s<SV FD:%s", fileName);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;     
    }   
    else { /* stores the data in the internal memory */     
        if (avr3132s_invalidViInt16Range (registerNumber, 0, 99))             
            return VI_ERROR_PARAMETER3;        
        Fmt (wrtBuffer,"%s<SV%d", registerNumber);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status; 
    }       
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Save File Options                                               */
/* Purpose:  determines which data will be saved                             */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Save_FileOptions (ViSession instrID, ViBoolean setupConditions,
                                         ViBoolean traceData, ViBoolean antennaCorrections,
                                         ViBoolean normCorr, ViBoolean traceLevel,
                                         ViInt16 limitLines, ViBoolean lossFreq,
                                         ViBoolean channel, ViBoolean spurious)
{
    ViStatus avr3132s_status = VI_SUCCESS; 
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    ViUInt32 total = 0;
    avr3132s_ptrFeatures featuresPtr;
   
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;

    if (avr3132s_invalidViBooleanRange (setupConditions))             
        return VI_ERROR_PARAMETER2;
    if (avr3132s_invalidViBooleanRange (traceData))             
        return VI_ERROR_PARAMETER3;
    if (avr3132s_invalidViBooleanRange (antennaCorrections))             
        return VI_ERROR_PARAMETER4;
    if (avr3132s_invalidViBooleanRange (normCorr))             
        return VI_ERROR_PARAMETER5;    
    if (avr3132s_invalidViBooleanRange (traceLevel))             
        return VI_ERROR_PARAMETER6;
    if (avr3132s_invalidViInt16Range (limitLines, 0, 3))             
        return VI_ERROR_PARAMETER7;
    if (avr3132s_invalidViBooleanRange (lossFreq))             
        return VI_ERROR_PARAMETER8;
    if (avr3132s_invalidViBooleanRange (channel))             
        return VI_ERROR_INSTR_PARAMETER9;
    if (avr3132s_invalidViBooleanRange (spurious))             
        return VI_ERROR_INSTR_PARAMETER10;
        
    Fmt (wrtBuffer, "%s<%s; %s; %s; %s; %s", s_saveItemSetup[setupConditions],
    	 s_saveItemTrcData[traceData], s_saveItemAntenna[antennaCorrections],
    	 s_saveItemNormCorr[normCorr], s_saveItemTrcLevel[traceLevel]);
    total += NumFmtdBytes();
    
    if (featuresPtr->instrumentType != 3131)
    {
    	Fmt (wrtBuffer, "%s[a]<; %s; %s; %s; %s", s_saveItemLmtLines[limitLines],
    		 s_saveItemLossFreq[lossFreq], s_saveItemChannel[channel],
    		 s_saveItemSpurious[spurious]);
    	total += NumFmtdBytes();	 
    }
    
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,total,&retCount))<0)
        return avr3132s_status;
        
    return avr3132s_status;    
}

/*===========================================================================*/
/* Function: recall a file                                                   */
/* Purpose:  selects the register or the file name to recall                 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Recall_File (ViSession instrID, ViBoolean drive,
                                                       ViInt16 registerNumber,
                                                       ViString fileName)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViBooleanRange (drive))             
        return VI_ERROR_PARAMETER2; 

    if (drive == VI_ON) { /* restores data from a floppy disk */
        if (CompareStrings(fileName,0,"",0,0)==0) return VI_ERROR_PARAMETER4;
        Fmt (wrtBuffer,"%s<RC FD:%s", fileName);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;     
    }   
    else { /* restores the data in the internal memory */       
        if (avr3132s_invalidViInt16Range (registerNumber, 0, 99))             
            return VI_ERROR_PARAMETER3;        
        Fmt (wrtBuffer,"%s<RC%d", registerNumber);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status; 
    }   
        
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: delete a file                                                   */
/* Purpose:  selects the register or the file name to delete                 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Delete_File (ViSession instrID, ViBoolean drive,
                                                       ViInt16 registerNumber,
                                                       ViString fileName)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViBooleanRange (drive))             
        return VI_ERROR_PARAMETER2; 

    if (drive == VI_ON) { /* deletes data on a floppy disk */
        if (CompareStrings(fileName,0,"",0,0)==0) return VI_ERROR_PARAMETER4;
        Fmt (wrtBuffer,"%s<DEL FD:%s", fileName);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;     
    }   
    else { /* deletes data in the internal memory */        
        if (avr3132s_invalidViInt16Range (registerNumber, 0, 99))             
            return VI_ERROR_PARAMETER3;        
        Fmt (wrtBuffer,"%s<DEL%d", registerNumber);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status; 
    }   
        
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Copy Bitmap (R3131)                                             */
/* Purpose:  allows to send screen data to a printer or to a floppy          */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Copy_Bitmap_R3131 (ViSession instrID,ViBoolean deviceSelection,
                                          ViInt16 printerSelection, ViBoolean copyMode,
                                          ViBoolean copySize)
{
    ViStatus avr3132s_status = VI_SUCCESS; 
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViBooleanRange (deviceSelection))             
        return VI_ERROR_PARAMETER2;
    
    if (deviceSelection==VI_ON) { /* -->printer */
        if (avr3132s_invalidViInt16Range (printerSelection,0,2))             
            return VI_ERROR_PARAMETER3;
        if (avr3132s_invalidViBooleanRange (copyMode))             
            return VI_ERROR_PARAMETER4;
        if (avr3132s_invalidViBooleanRange (copySize))             
            return VI_ERROR_PARAMETER5;
        Fmt (wrtBuffer,"%s<%s,%s,%s; HCDEV PRT; HCOPY",s_printerR3131[printerSelection],
             s_prtModeR3131[copyMode], s_printSize[copySize]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
    else { /* -->floppy */
        Fmt (wrtBuffer,"%s<HCDEV FDD; HCOPY");   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }       
       
    return avr3132s_status;
}

/*======================================================================*/
/* Function: Set Market Antenna Correction                              */
/* Purpose:  Activates or deactivates the antenna correction function   */
/*           for the antenna TR1722, UHALP9107, BBA9106 and EMC03142    */
/*======================================================================*/ 
ViStatus _VI_FUNC avr3132s_Set_MarketAntennaCorrection (ViSession instrID, ViInt16 antennaChoice)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViInt16Range (antennaChoice,0,4))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_antennaAC[antennaChoice]);   
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;    
        
    return avr3132s_status;    
}        

/*===========================================================================*/
/* Function: Copy Bitmap (R3132s)                                            */
/* Purpose:  allows to send screen data to a printer or to a floppy          */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Copy_Bitmap_R3132s (ViSession instrID, 
										   ViBoolean deviceSel, ViInt16 printerSel,
										   ViInt16 prtMode, ViInt16 bmpMode,
										   ViBoolean fileCmprss, ViInt16 fileNum)
{
    ViStatus avr3132s_status = VI_SUCCESS; 
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViBooleanRange (deviceSel))             
        return VI_ERROR_PARAMETER2;
        
    if (deviceSel==VI_ON) /* Printer */
    {
    	if (avr3132s_invalidViInt16Range (printerSel, 0, 2))             
        	return VI_ERROR_PARAMETER3;
        if (avr3132s_invalidViInt16Range (prtMode, 0, 5))             
        	return VI_ERROR_PARAMETER4;
        
        Fmt (wrtBuffer, "%s<%s; %s; HCDEV PRT; HCOPY", s_printerR3132s[printerSel], s_prtModeR3132s[prtMode]);
        
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        	return avr3132s_status;
    }
    
    else /* Floppy */
    {
    	if (avr3132s_invalidViInt16Range (bmpMode, 0, 3))             
        	return VI_ERROR_PARAMETER5;
        if (avr3132s_invalidViBooleanRange (fileCmprss))             
        	return VI_ERROR_PARAMETER6;
        if (avr3132s_invalidViInt16Range (fileNum, 0, 999))             
        	return VI_ERROR_PARAMETER7;
        	
        Fmt (wrtBuffer, "%s<%s; %s; HCFILE %d; HCDEV FDD; HCOPY",
        	s_BMPMode[bmpMode], s_fileCompress[fileCmprss], fileNum);
        
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        	return avr3132s_status;	 
    }
    
    return avr3132s_status;
}

/*======================================================================*/
/* Function: Set User Antenna Correction                                */
/* Purpose:  Activates or deactivates the antenna correction function   */
/*           Can be used whatever the antenna is                        */
/*======================================================================*/
ViStatus _VI_FUNC avr3132s_Set_UserAntennaCorrection (ViSession instrID, 
												  ViBoolean status, ViBoolean corrType,
                                                  ViReal64 freqTable[], ViReal64 levelTable[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 i=0;
    
    if (avr3132s_invalidViBooleanRange (status))             
        return VI_ERROR_PARAMETER2;
        
    if (avr3132s_invalidViBooleanRange (corrType))             
        return VI_ERROR_PARAMETER3;    
        
    if (status == VI_ON) {  /* Antenna Correction ON */
      
        Fmt (wrtBuffer,"%s<%s\r\n",s_workAC[0]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
         
        i=0;
        while ( (i<=24)&&((freqTable[i]!=0)||(freqTable[i+1]!=0)||(freqTable[i+2]!=0)) ) {
            Fmt (wrtBuffer,"%s<%s%fHZ,%fDB\r\n",s_workAC[1],freqTable[i],levelTable[i]);   
            if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
                return avr3132s_status;
            i++;   
        }
        Fmt (wrtBuffer,"%s<%s,%s\r\n",s_modeAC[corrType],s_showAC[status]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
        
    }      
    else {    /* Antenna Correction OFF */     
        Fmt (wrtBuffer,"%s<%s\r\n",s_showAC[status]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status; 
    
    }       
     
    return avr3132s_status; 
}

/*===========================================================================*/
/* Function: Read Antenna Correction Status                                  */
/* Purpose:  returns the level correction  status                            */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_AntennaCorrectionStatus (ViSession instrID,ViPInt16 status,
                                                                       ViChar _VI_FAR message[])
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    ViInt16 corrtype;
    
    Fmt (Buffer,"%s<%s\r\n",s_readAC[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    if (*status == 5) { 
        Fmt (Buffer,"%s<%s\r\n",s_readAC[1]);   
        if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    
        if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
            return avr3132s_status;
       
        if (Scan(Buffer,"%s>%d[b2]", &corrtype)!=1) { 
            avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            return avr3132s_status;
        }
        *status = *status + corrtype;
    }
    
    Fmt(message,"%s<%s",s_readAC[*status+2]); 
    
    return avr3132s_status;
} 

/*===========================================================================*/
/* Function: Set Tracking Generator status                                   */           
/* Purpose:  Specifies the TG level and activate or deactivate it            */
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_TGstatus (ViSession instrID, ViBoolean status, ViReal64 level)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    
    if (avr3132s_invalidViBooleanRange (status))             
        return VI_ERROR_PARAMETER2;
    
    if (status == VI_ON) { /* activation */
        Fmt (wrtBuffer,"%s<%s,%s%f%s\r\n",s_setTGstatus[status], s_setTGstatus[2], 
        								  level, s_levelUnit[featuresPtr->refUnit]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    }
    else {  /* deactivation */
        Fmt (wrtBuffer,"%s<%s\r\n",s_setTGstatus[status]);   
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
    } 

    return avr3132s_status;                                          
}

/*===========================================================================*/
/* Function: Read Tracking Generator Status                                  */
/* Purpose:  returns the output level and the status of the TG               */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_TGstatus (ViSession instrID, ViPInt16 status,
                                      					 ViChar _VI_FAR message[],
                                      					 ViPReal64 level)	
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    /* returns the TG status */
    
    Fmt (Buffer,"%s<%s\r\n",s_readTGstatus[0]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    Fmt(message,"%s<%s",s_textONOFF[*status]);
    
    /* returns the TG output level */
    
    Fmt (Buffer,"%s<%s\r\n",s_readTGstatus[1]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
        
    if (Scan(Buffer,"%s>%f", level)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Set TG Frequency Compensation                                   */           
/* Purpose:  set the tracking generator frequency compensation               */           
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_TGfreqComp (ViSession instrID, ViBoolean compType,
										   				  ViReal64 freqCompValue)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((avr3132s_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;

    if (avr3132s_invalidViBooleanRange (compType))             
        return VI_ERROR_PARAMETER2;
       
    if (compType == VI_ON) { /*frequency compensation made by the analyzer*/ 
        Fmt (wrtBuffer,"%s<%s\r\n", s_TGfreqComp[compType]);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;
    }
    else { /*frequency compensation chosen by the user*/   
        if (avr3132s_invalidViReal64Range (freqCompValue, -500.0e3, 500.0e3))             
            return VI_ERROR_PARAMETER3;   
        Fmt (wrtBuffer,"%s<%s%fHZ\r\n",s_TGfreqComp[compType],freqCompValue);     
        if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return avr3132s_status;
    }
     
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read TG Frequency Compensation                                  */           
/* Purpose:  returns the tracking generator frequency compensation           */           
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Read_TGfreqComp (ViSession instrID, ViPReal64 TGfreqComp)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_TGfreqComp[2]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f", TGfreqComp)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
     
    return avr3132s_status;
}								

/*===========================================================================*/
/* Function: Set TG normalization function                                   */           
/* Purpose:  allows to use the TG normalization function                     */           
/*===========================================================================*/           
ViStatus _VI_FUNC avr3132s_Set_TGnormFunction (ViSession instrID,ViInt16 normChoice)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (avr3132s_invalidViInt16Range (normChoice, 0, 2))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_TGnormFunction[normChoice]);     
    if ((avr3132s_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Read TG normalization function                                  */
/* Purpose:  returns the status of the TG normalization function             */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_Read_TGnormFunction (ViSession instrID, ViPInt16 status,
                                      					 	   ViChar _VI_FAR message[])	
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    Fmt (Buffer,"%s<%s\r\n",s_TGnormFunction[3]);   
    if ((avr3132s_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    
    if ((avr3132s_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return avr3132s_status;
    
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    Fmt(message,"%s<%s",s_textONOFF[*status]);
    
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC avr3132s_close (ViSession instrSession)
{
    avr3132s_ptrFeatures featuresPtr;
    ViSession rmSession;
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if ((avr3132s_status = viGetAttribute (instrSession,VI_ATTR_RM_SESSION,&rmSession))<0)
        return avr3132s_status;
    if ((avr3132s_status = viGetAttribute (instrSession,VI_ATTR_USER_DATA,&featuresPtr))<0)
        return avr3132s_status;
    
    if (featuresPtr != NULL) 
        free (featuresPtr);
    
    avr3132s_status = viClose (instrSession);
    viClose (rmSession);

    return avr3132s_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/


/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean avr3132s_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_OFF && val != VI_ON) ? VI_ON : VI_OFF);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avr3132s_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return    */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avr3132s_invalidViInt32Range  (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Unsigned Integer Value Out Of Range - ViUInt16            */
/* Purpose:  This function checks a short unsigned integer value to see if it*/  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return    */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avr3132s_invalidViUInt16Range  (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return    */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avr3132s_invalidViUInt32Range  (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}
/*===========================================================================*/
/* Function: Real (Float) Value Out Of Range - ViReal32                      */
/* Purpose:  This function checks a real (float) value to see if it lies     */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avr3132s_invalidViReal32Range  (ViReal32 val, ViReal32 min, ViReal32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avr3132s_invalidViReal64Range  (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the avr3132s_init function.  When     */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from avr3132s_init to         */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus avr3132s_initCleanUp (ViSession openRMSession,
                           ViPSession openInstrSession, ViStatus currentStatus)
{
    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
    
}

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* function  Search Scale                                                    */
/* Purpose : search the scale used by the analyzer at the beginning of the   */
/*           remote control                                                  */
/*===========================================================================*/
ViStatus avr3132s_Search_Scale (ViSession instrSession, ViPInt16 scale)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    ViInt16   i, unitNumberStart, unitNumber;
    ViChar unitName[5];
    
    /* returns the unit */
    avr3132s_Read_RefUnit (instrSession, &unitNumberStart, unitName);
    
    /* send a new unit --> dBm */
    avr3132s_Set_RefUnit (instrSession, 0);
    
    /* we send the order to pass in linear scale */
    /* if we are in log scale --> lin scale and unit = V*/
    /* if we are in lin scale --> lin scale and unit = dBm*/
    Fmt (Buffer,"%s<%s\r\n",s_yScale[4]);   
    if ((avr3132s_status = viWrite(instrSession,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    avr3132s_Read_RefUnit (instrSession, &unitNumber, unitName);
    if (unitNumber == 5) {  /* unit = V --> log scale */ 
        Fmt (Buffer,"%s<%s\r\n",s_yScale[5]);   
        if ((avr3132s_status = viWrite(instrSession,Buffer,NumFmtdBytes(),&retCount))<0)
            return avr3132s_status;
        if ((avr3132s_status = viRead (instrSession, Buffer, 5, &retCount)) < 0)
            return avr3132s_status;
        if (Scan(Buffer,"%s>%d[b2]", scale)!=1) { 
            avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            return avr3132s_status;
        }
        Fmt (Buffer,"%s<%s\r\n",s_yScale[*scale]);   
        if ((avr3132s_status = viWrite(instrSession,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    }
    else {  /* unit = dBm --> linear scale */
        *scale = 4;
    }
    
    /* send the unit used at the beginning of this function */
    avr3132s_Set_RefUnit (instrSession, unitNumberStart);
        
    return avr3132s_status;
} 

/*===========================================================================*/
/* function  Search Reference Level Offset Status                            */
/* Purpose : search whether the ref offset is active or not at the beginning */
/*           of the remote control                                           */
/*===========================================================================*/
ViStatus avr3132s_Search_RefOffsetStatus (ViSession instrSession, ViPInt16 refOffsetStatus)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    ViReal64 refOffsetStart, refLevelStart, refLevelAfter;
    ViChar unitName[5];
    avr3132s_ptrFeatures featuresPtr;
    
    if ((avr3132s_status = viGetAttribute (instrSession,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return avr3132s_status;
    
    /* reads the value of the reference offset */
    Fmt (Buffer,"%s<%s\r\n",s_readRefOffset[0]);   
    if ((avr3132s_status = viWrite(instrSession,Buffer,NumFmtdBytes(),&retCount))<0)
        return avr3132s_status;
    if ((avr3132s_status = viRead (instrSession, Buffer, 25, &retCount)) < 0)
        return avr3132s_status;
    if (Scan(Buffer,"%s>%f", &refOffsetStart)!=1) { 
        avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return avr3132s_status;
    }
    
    
    if (featuresPtr->instrumentType==3131) {
    
    	/* reads the value of the reference level */
    	avr3132s_Read_RefLevel (instrSession, &refLevelStart);
    
	    if (refOffsetStart == 0) {
	        *refOffsetStatus = 0;
	        /* deactivate the reference offset if it was active */
	        Fmt (Buffer,"%s<%s\r\n", s_refOffset[0]);     
	        if ((avr3132s_status = viWrite(instrSession,Buffer,NumFmtdBytes(),&retCount)) < 0)
	            return avr3132s_status;
	    }
	    else {
	        /* deactivate the reference offset */
	        Fmt (Buffer,"%s<%s\r\n", s_refOffset[0]);     
	        if ((avr3132s_status = viWrite(instrSession,Buffer,NumFmtdBytes(),&retCount)) < 0)
	            return avr3132s_status;
	        /* reads the new value of the reference level after the deactivation of the ref offset*/
	        avr3132s_Read_RefLevel (instrSession, &refLevelAfter);
	        /* status of the reference offset */
	        if (refLevelAfter != refLevelStart) {  /*ref offset was active*/
	            *refOffsetStatus = 1;
	            Fmt (Buffer,"%s<%s%fDB\r\n", s_refOffset[2], refOffsetStart);     
	            if ((avr3132s_status = viWrite(instrSession,Buffer,NumFmtdBytes(),&retCount)) < 0)
	                return avr3132s_status;
	        }
	        else {  /* reference offset was inactive */
	            *refOffsetStatus = 0;
	        }
	    }
	    
	 }
	 
	 else {
	 	
	 	Fmt (Buffer,"%s<%s\r\n",s_readRefOffset[1]);   
    	if ((avr3132s_status = viWrite(instrSession,Buffer,NumFmtdBytes(),&retCount))<0)
        	return avr3132s_status;
    	if ((avr3132s_status = viRead (instrSession, Buffer, 25, &retCount)) < 0)
        	return avr3132s_status;
    	if (Scan(Buffer,"%s>%d[b2]", refOffsetStatus)!=1) { 
        	avr3132s_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        	return avr3132s_status;
    	}
    	
	 }	
	 
         
    return avr3132s_status;
} 

/*===========================================================================*/
/* Function: Internal Reset                                                  */
/* Purpose:  This function resets the instrument.                            */
/*===========================================================================*/
ViStatus avr3132s_initReset (ViSession instrSession,ViUInt32 instrType)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    /*  Initialize the instrument to a known state.  */
    Fmt (wrtBuffer,"%s<%s\r\n",s_reset[0]);     
    if ((avr3132s_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return avr3132s_status;
   
    if ((avr3132s_status = avr3132s_defaultInstrSetup (instrSession,instrType)) < 0)  
        return avr3132s_status;
        
    return avr3132s_status;
}

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the avr3132s_reset operation and by the       */
/*           avr3132s_init function if the reset option has not been             */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus avr3132s_defaultInstrSetup (ViSession instrSession, ViUInt32 instrType)
{
    ViStatus avr3132s_status = VI_SUCCESS;
    ViUInt32 retCount = 0;
    ViByte wrtBuffer[BUFFER_SIZE];
    avr3132s_ptrFeatures featuresPtr;
    ViInt16 i;
    ViReal64 MKRMpositions[10],positionDelta,MKRMlevels[10];
    ViReal64 levelDelta, span;
    ViChar unitName[10];
    
    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    if (avr3132s_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &featuresPtr))
        return avr3132s_status;
    
    if (featuresPtr == NULL) 
        featuresPtr = malloc (sizeof (struct avr3132s_instrFeatures));
        
   if (viSetAttribute (instrSession, VI_ATTR_USER_DATA, (ViUInt32)featuresPtr)!= 0)
       return avr3132s_status;
    
    avr3132s_Read_RefUnit (instrSession, &featuresPtr->refUnit,unitName);         
        
    switch (instrType) {
    
    	case 3131 :
            featuresPtr->centerLow = 0.0;
            featuresPtr->centerHigh = 3.0e9;
            featuresPtr->startLow = -200.0e6;
            featuresPtr->startHigh = 3.0e9;
            featuresPtr->stopLow = 0.0;
            featuresPtr->stopHigh = 3.2e9;
            featuresPtr->spanLow = 1.0e3;
            featuresPtr->spanHigh = 3.2e9;
            featuresPtr->sweepTimeMini = 50.0e-6;
            featuresPtr->sweepTimeMaxi = 800.0;
            featuresPtr->inputAttMin = 0.0;   
    		featuresPtr->inputAttMax = 50.0;  
            break;
            
        case 3132 :
            featuresPtr->centerLow = 0.0;
            featuresPtr->centerHigh = 3.0e9;
            featuresPtr->startLow = -200.0e6;
            featuresPtr->startHigh = 3.0e9;
            featuresPtr->stopLow = 0.0;
            featuresPtr->stopHigh = 3.2e9;
            featuresPtr->spanLow = 1.0e3;
            featuresPtr->spanHigh = 3.2e9;
            featuresPtr->freqOffsetMin = -100.0e9; 
   			featuresPtr->freqOffsetMax = +100.0e9;
            featuresPtr->sweepTimeMini = 50.0e-6;
            featuresPtr->sweepTimeMaxi = 800.0;
            featuresPtr->inputAttMin = 0.0;   
    		featuresPtr->inputAttMax = 50.0;  
            break;
            
        case 3162 :
            featuresPtr->centerLow = 0.0;
            featuresPtr->centerHigh = 8.0e9;
            featuresPtr->startLow = -200.0e6;
            featuresPtr->startHigh = 8.0e9;
            featuresPtr->stopLow = 0.0;
            featuresPtr->stopHigh = 8.2e9;
            featuresPtr->spanLow = 1.0e3;
            featuresPtr->spanHigh = 8.2e9;
            featuresPtr->freqOffsetMin = -100.0e9; 
    		featuresPtr->freqOffsetMax = +100.0e9;
            featuresPtr->sweepTimeMini = 50.0e-6;
            featuresPtr->sweepTimeMaxi = 800.0;
            featuresPtr->inputAttMin = 0.0; 
    		featuresPtr->inputAttMax = 75.0;
            break;
            
            
        case 3172 :
            featuresPtr->centerLow = 0.0;
            featuresPtr->centerHigh = 27.0e9;
            featuresPtr->startLow = -200.0e6;
            featuresPtr->startHigh = 27.0e9;
            featuresPtr->stopLow = 0.0;
            featuresPtr->stopHigh = 27.0e9;
            featuresPtr->spanLow = 1.0e3;
            featuresPtr->spanHigh = 27.0e9;
            featuresPtr->freqOffsetMin = -100.0e9; 
    		featuresPtr->freqOffsetMax = +100.0e9;
            featuresPtr->sweepTimeMini = 50.0e-6;
            featuresPtr->sweepTimeMaxi = 800.0;
            featuresPtr->inputAttMin = 0.0; 
    		featuresPtr->inputAttMax = 75.0;
            break;
            
            
        case 3182 :
            featuresPtr->centerLow = 0.0;
            featuresPtr->centerHigh = 42.0e9;
            featuresPtr->startLow = -200.0e6;
            featuresPtr->startHigh = 42.0e9;
            featuresPtr->stopLow = 0.0;
            featuresPtr->stopHigh = 42.0e9;
            featuresPtr->spanLow = 1.0e3;
            featuresPtr->spanHigh = 42.0e9;
            featuresPtr->freqOffsetMin = -100.0e9; 
    		featuresPtr->freqOffsetMax = +100.0e9;
            featuresPtr->sweepTimeMini = 50.0e-6;
            featuresPtr->sweepTimeMaxi = 800.0;
            featuresPtr->inputAttMin = 0.0; 
    		featuresPtr->inputAttMax = 75.0;
            break;    
    }
    
    featuresPtr->instrumentType = instrType;
    featuresPtr->refLevelOffsetMin = -100; /*dB*/
    featuresPtr->refLevelOffsetMax = +100;
    featuresPtr->averageCountMini = 2;
    featuresPtr->averageCountMaxi = 1000;
    featuresPtr->DLlevelMin = -1000;
    featuresPtr->DLlevelMax = 1000;
    featuresPtr->deltaYmin=0.1;
    featuresPtr->deltaYmax=10;
    featuresPtr->calLevelMin=-80.0;
    featuresPtr->calLevelMax=0.0;

    /* checks whether the span is null or not */
    avr3132s_Read_FreqSettings (instrSession, 1, &span);
    if (span==0) featuresPtr->span = 0;
    else featuresPtr->span = 1;
    
    avr3132s_Read_Multimarkers (instrSession,MKRMpositions,&positionDelta,MKRMlevels,&levelDelta);
                                         
    for (i=0;i<10;i++) {
        if ((MKRMpositions[i]!=0)||(MKRMlevels[i]!=0)) featuresPtr->MKRMstatus[i]=1;
        else featuresPtr->MKRMstatus[i]=0;
    }
    
    Fmt (featuresPtr -> instrDriverRevision, "%s<%s", AVR3132S_REVISION);    
    
    return avr3132s_status;
    
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
